@@ -27,7 +27,7 @@ Get all the entries, both read and unread::
 
 Mark the first entry as read::
 
-    reader.mark_as_read(entries[0])
+    reader.mark_entry_as_read(entries[0])
 
 Print the titles of the unread entries::
 
@@ -37,7 +37,7 @@ Print the titles of the unread entries::
 
 """
 
-__version__ = '2.0'
+__version__ = '2.5'
 
 from .core import Reader, make_reader
 
@@ -63,7 +63,9 @@ from .exceptions import (
     FeedExistsError,
     FeedNotFoundError,
     ParseError,
+    InvalidFeedURLError,
     EntryError,
+    EntryExistsError,
     EntryNotFoundError,
     MetadataError,
     MetadataNotFoundError,
