@@ -1,6 +1,7 @@
 import contextlib
 import itertools
 import json
+import math
 import time
 from dataclasses import dataclass
 from datetime import datetime
@@ -46,6 +47,16 @@ def humanize_naturaltime(dt):
     return humanize.naturaltime(dt, when=when)
 
 
+@blueprint.app_template_filter()
+def log_scale(n, p=2):
+    # https://github.com/lemon24/reader/issues/249#issuecomment-893440484
+    # https://math.stackexchange.com/a/970251
+    # https://math.stackexchange.com/a/3428961
+    n = float(n)
+    c = 10 ** p
+    return math.log10(n * c + 1) / math.log10(c + 1)
+
+
 # if any plugins need signals, they need to install blinker
 signals = flask.signals.Namespace()
 
@@ -253,6 +264,10 @@ def entries():
     if feed_url:
         entries_data = [e.id for e in entries]
 
+    feed_entry_counts = None
+    if feed_url:
+        feed_entry_counts = reader.get_entry_counts(feed=feed)
+
     # Ensure flashed messages get removed from the session,
     # otherwise they keep adding up and never disappear.
     # Assumes the template will call get_flashed_messages() at some point.
@@ -267,6 +282,7 @@ def entries():
         entries_data=entries_data,
         error=error,
         counts=counts,
+        feed_entry_counts=feed_entry_counts,
     )
 
 
@@ -283,7 +299,7 @@ def preview():
         'default', url=':memory:', plugin_loader=current_app.plugin_loader
     )
 
-    reader.add_feed(url)
+    reader.add_feed(url, allow_invalid_url=True)
 
     try:
         reader.update_feed(url)
@@ -298,9 +314,31 @@ def preview():
 
     feed = reader.get_feed(url)
     entries = list(reader.get_entries())
+    feed_entry_counts = reader.get_entry_counts(feed=url)
 
     # TODO: maybe limit
-    return stream_template('entries.html', entries=entries, feed=feed, read_only=True)
+    return stream_template(
+        'entries.html',
+        entries=entries,
+        feed=feed,
+        read_only=True,
+        feed_entry_counts=feed_entry_counts,
+    )
+
+
+@blueprint.route('/add-entry')
+def add_entry():
+    reader = get_reader()
+
+    feed_url = request.args['feed']
+    feed = reader.get_feed(feed_url, None)
+    if not feed:
+        abort(404)
+
+    return render_template(
+        'add_entry.html',
+        feed=feed,
+    )
 
 
 @blueprint.route('/feeds')
@@ -435,11 +473,11 @@ def readererror_to_apierror(*args):
         yield
     except ReaderError as e:
         category = None
-        if hasattr(e, 'url'):
-            category = (e.url,)
-            if hasattr(e, 'id'):
-                category += (e.id,)
-        raise APIError(str(e), category)
+        if hasattr(e, 'object_id'):
+            category = e.object_id
+            if not isinstance(category, tuple):
+                category = (category,)
+        raise APIError(str(e), category) from e
 
 
 @form_api
@@ -455,7 +493,21 @@ def mark_as_read(data):
 def mark_as_unread(data):
     feed_url = data['feed-url']
     entry_id = data['entry-id']
-    get_reader().mark_entry_as_unread((feed_url, entry_id))
+    id = feed_url, entry_id
+    reader = get_reader()
+    reader.mark_entry_as_unread(id)
+    entry = reader.get_entry(id)
+    # if we're in "don't care", reset important_modified
+    if not entry.important and entry.important_modified:
+        reader.set_entry_important(id, False, modified=None)
+
+
+@form_api
+@readererror_to_apierror()
+def mark_as_dont_care(data):
+    feed_url = data['feed-url']
+    entry_id = data['entry-id']
+    get_reader()._mark_entry_as_dont_care((feed_url, entry_id))
 
 
 @form_api(really=True)
@@ -489,7 +541,8 @@ def mark_as_important(data):
 def mark_as_unimportant(data):
     feed_url = data['feed-url']
     entry_id = data['entry-id']
-    get_reader().mark_entry_as_unimportant((feed_url, entry_id))
+    # only "don't care" sets important_modified
+    get_reader().set_entry_important((feed_url, entry_id), False, modified=None)
 
 
 @form_api(really=True)
@@ -532,11 +585,11 @@ def update_metadata(data):
     try:
         value = yaml.safe_load(data['value'])
     except yaml.YAMLError as e:
-        raise APIError("invalid JSON: {}".format(e), (feed_url, key))
+        raise APIError("invalid JSON: {}".format(e), (feed_url, key)) from e
     get_reader().set_feed_metadata_item(feed_url, key, value)
 
 
-@form_api
+@form_api(really=True)
 @readererror_to_apierror()
 def delete_metadata(data):
     feed_url = data['feed-url']
@@ -593,6 +646,29 @@ def update_feed(data):
     get_reader().update_feed(feed_url)
 
 
+@form_api(name='add-entry')
+@readererror_to_apierror()
+def add_entry_action(data):
+    feed_url = data['feed-url']
+    link = data['link'].strip()
+    title = data['title'].strip() or None
+    if not link:
+        raise APIError(f"invalid link: {link}", (feed_url,))
+
+    reader = get_reader()
+
+    # TODO: if we ever add other stuff, we should catch the various AttributeError/ValueError/TypeError from the conversion here
+    reader.add_entry(dict(feed_url=feed_url, id=link, link=link, title=title))
+
+
+@form_api(really=True)
+@readererror_to_apierror()
+def delete_entry(data):
+    feed_url = data['feed-url']
+    entry_id = data['entry-id']
+    get_reader().delete_entry((feed_url, entry_id))
+
+
 # for some reason, @blueprint.app_template_global does not work
 @blueprint.app_template_global()
 def additional_enclosure_links(enclosure, entry):
