@@ -4,6 +4,17 @@ import reader
 from reader._cli import setup_logging
 
 
+def make_add_response_headers_middleware(wsgi_app, headers):
+    def wsgi_app_wrapper(environ, start_response):
+        def start_response_wrapper(status, response_headers, exc_info=None):
+            response_headers.extend(headers)
+            return start_response(status, response_headers, exc_info)
+
+        return wsgi_app(environ, start_response_wrapper)
+
+    return wsgi_app_wrapper
+
+
 @click.command()
 @click.pass_obj
 @click.option('-h', '--host', default='localhost', help="The interface to bind to.")
@@ -28,4 +39,9 @@ def serve(config, host, port, plugin, verbose):
     config['default']['reader'].pop('debug_storage', None)
 
     app = create_app(config)
+    app.wsgi_app = make_add_response_headers_middleware(
+        app.wsgi_app,
+        [('Referrer-Policy', 'same-origin')],
+    )
+
     run_simple(host, port, app)
