@@ -45,6 +45,7 @@ from ._url_utils import extract_path
 from ._url_utils import is_windows_device_file
 from ._url_utils import normalize_url
 from ._url_utils import resolve_root
+from .exceptions import InvalidFeedURLError
 from .exceptions import ParseError
 from .types import Content
 from .types import Enclosure
@@ -74,9 +75,17 @@ class RetrieverType(Protocol):
     ) -> ContextManager[Optional[RetrieveResult]]:  # pragma: no cover
         ...
 
+    def validate_url(self, url: str) -> None:
+        """Check if ``url`` is valid for this retriever.
+
+        Raises:
+            InvalidFeedURLError: If ``url`` is not valid.
+
+        """
+
 
 # TODO: use Collection instead of Iterable
-FeedAndEntries = Tuple[FeedData, Iterable[EntryData[Optional[datetime]]]]
+FeedAndEntries = Tuple[FeedData, Iterable[EntryData]]
 
 
 class ParserType(Protocol):
@@ -188,6 +197,22 @@ class Parser:
 
         return ParsedFeed(feed, entries, result.http_etag, result.http_last_modified)
 
+    def validate_url(self, url: str) -> None:
+        """Check if ``url`` is valid without actually retrieving it.
+
+        Raises:
+            InvalidFeedURLError: If ``url`` is not valid.
+
+        """
+        try:
+            retriever = self.get_retriever(url)
+        except ParseError as e:
+            raise InvalidFeedURLError(e.url, message=e.message) from None
+        try:
+            retriever.validate_url(url)
+        except ValueError as e:
+            raise InvalidFeedURLError(url) from e
+
     def mount_retriever(self, prefix: str, retriever: RetrieverType) -> None:
         self.retrievers[prefix] = retriever
         keys_to_move = [k for k in self.retrievers if len(k) < len(prefix)]
@@ -288,6 +313,9 @@ class FileRetriever:
             with open(normalized_url, 'rb') as file:
                 yield RetrieveResult(file)
 
+    def validate_url(self, url: str) -> None:
+        self._normalize_url(url)
+
     def _normalize_url(self, url: str) -> str:
         path = extract_path(url)
         if self.feed_root:
@@ -385,6 +413,11 @@ class HTTPRetriever:
                     response_headers,
                 )
 
+    def validate_url(self, url: str) -> None:
+        session = self.make_session().session
+        session.get_adapter(url)
+        session.prepare_request(requests.Request('GET', url))
+
 
 def _caching_get(
     session: SessionWrapper,
@@ -473,6 +506,8 @@ def _process_feedparser_dict(url: str, d: Any) -> FeedAndEntries:
         d.feed.get('title'),
         d.feed.get('link'),
         d.feed.get('author'),
+        d.feed.get('subtitle') or None,
+        d.version,
     )
     # This must be a list, not a generator expression,
     # otherwise the user may get a ParseError when calling
@@ -494,9 +529,6 @@ def _get_updated_published(
     updated = convert('updated_parsed')
     published = convert('published_parsed')
 
-    if published and not updated and is_rss:
-        updated, published = published, None
-
     return updated, published
 
 
@@ -504,9 +536,7 @@ def _datetime_from_timetuple(tt: time.struct_time) -> datetime:
     return datetime.utcfromtimestamp(calendar.timegm(tt))
 
 
-def _feedparser_entry(
-    feed_url: str, entry: Any, is_rss: bool
-) -> EntryData[Optional[datetime]]:
+def _feedparser_entry(feed_url: str, entry: Any, is_rss: bool) -> EntryData:
     id = entry.get('id')
 
     # <guid> (entry.id) is not actually required for RSS;
@@ -575,10 +605,20 @@ class JSONFeedParser:
         return _process_jsonfeed_dict(url, result)
 
 
+_JSONFEED_VERSIONS = {
+    "https://jsonfeed.org/version/1.1": 'json11',
+    "https://jsonfeed.org/version/1": 'json10',
+}
+_JSONFEED_VERSION_URL_PREFIX = "https://jsonfeed.org/version/"
+_JSONFEED_VERSION_UNKNOWN = 'json'
+
+
 def _process_jsonfeed_dict(url: str, d: Any) -> FeedAndEntries:
     version = _dict_get(d, 'version', str) or ''
-    if not version.lower().startswith('https://jsonfeed.org/version/'):
-        raise ParseError(url, "missing or bad JSON Feed version")
+    version_lower = version.lower()
+    if not version_lower.startswith(_JSONFEED_VERSION_URL_PREFIX):
+        raise ParseError(url, f"missing or bad JSON Feed version: {version!r}")
+    version_code = _JSONFEED_VERSIONS.get(version_lower, _JSONFEED_VERSION_UNKNOWN)
 
     feed = FeedData(
         url=url,
@@ -586,6 +626,8 @@ def _process_jsonfeed_dict(url: str, d: Any) -> FeedAndEntries:
         title=_dict_get(d, 'title', str),
         link=_dict_get(d, 'home_page_url', str),
         author=_jsonfeed_author(d),
+        subtitle=_dict_get(d, 'description', str),
+        version=version_code,
     )
     lang = _dict_get(d, 'language', str)
 
@@ -643,9 +685,7 @@ def _jsonfeed_author(d: Any) -> Optional[str]:
     )
 
 
-def _jsonfeed_entry(
-    feed_url: str, d: Any, feed_lang: Optional[str]
-) -> EntryData[Optional[datetime]]:
+def _jsonfeed_entry(feed_url: str, d: Any, feed_lang: Optional[str]) -> EntryData:
     updated_str = _dict_get(d, 'date_modified', str)
     updated = _parse_jsonfeed_date(updated_str) if updated_str else None
     published_str = _dict_get(d, 'date_published', str)
