@@ -8,7 +8,6 @@ import warnings
 from collections import OrderedDict
 from contextlib import contextmanager
 from datetime import datetime
-from datetime import timedelta
 from functools import partial
 from itertools import groupby
 from types import MappingProxyType
@@ -23,6 +22,8 @@ from typing import TYPE_CHECKING
 from typing import TypeVar
 from typing import Union
 
+import bs4  # type: ignore
+
 from ._sql_utils import paginated_query
 from ._sql_utils import Query
 from ._sqlite_utils import DBError
@@ -34,6 +35,7 @@ from ._sqlite_utils import wrap_exceptions_iter
 from ._storage import apply_entry_filter_options
 from ._storage import apply_random
 from ._storage import apply_recent
+from ._storage import make_entry_counts_query
 from ._storage import Storage
 from ._types import EntryFilterOptions
 from ._utils import exactly_one
@@ -52,15 +54,6 @@ if TYPE_CHECKING:  # pragma: no cover
     from ._storage import Storage  # noqa: F401
 
 
-# Only Search.update() has a reason to fail if bs4 is missing.
-try:
-    import bs4  # type: ignore
-
-    bs4_import_error = None
-except ImportError as e:  # pragma: no cover
-    bs4 = None
-    bs4_import_error = e
-
 log = logging.getLogger('reader')
 
 
@@ -123,20 +116,19 @@ _QUERY_ERROR_MESSAGE_FRAGMENTS = [
 
 
 @contextmanager
-def wrap_search_exceptions() -> Iterator[None]:
+def wrap_search_exceptions(enabled: bool = True, query: bool = False) -> Iterator[None]:
     try:
         yield
     except sqlite3.OperationalError as e:
         msg_lower = str(e).lower()
 
-        if 'no such table' in msg_lower:
-            raise SearchNotEnabledError()
+        if enabled and 'no such table' in msg_lower:
+            raise SearchNotEnabledError() from None
 
-        is_query_error = any(
+        if query and any(
             fragment in msg_lower for fragment in _QUERY_ERROR_MESSAGE_FRAGMENTS
-        )
-        if is_query_error:
-            raise InvalidSearchQueryError(message=str(e))
+        ):
+            raise InvalidSearchQueryError(message=str(e)) from None
 
         raise
 
@@ -144,6 +136,7 @@ def wrap_search_exceptions() -> Iterator[None]:
 # When trying to fix "database is locked" errors or to optimize stuff,
 # have a look at the lessons here first:
 # https://github.com/lemon24/reader/issues/175#issuecomment-657495233
+# tl;dr: Measure. Measure in prod. FTS5 tables are slow for non-FTS queries.
 
 # When adding a new method, add a new test_search.py::test_errors_locked test.
 
@@ -186,38 +179,46 @@ class Search:
 
     """
 
-    def __init__(self, storage: Union['Storage', sqlite3.Connection]):
+    def __init__(self, storage: Union[Storage, sqlite3.Connection]):
         # during migrations there's no storage, so we need to pass the DB in
-        self.storage = storage
+        self._storage = storage
 
-    # db, recent_threshold, and chunk_size exposed for convenience.
+    # db, storage, and chunk_size exposed for (typing) convenience.
 
     @property
     def db(self) -> sqlite3.Connection:
-        if isinstance(self.storage, sqlite3.Connection):  # pragma: no cover
-            return self.storage
-        return self.storage.db
+        if isinstance(self._storage, sqlite3.Connection):  # pragma: no cover
+            return self._storage
+        return self._storage.db
 
     @property
-    def recent_threshold(self) -> timedelta:
-        if isinstance(self.storage, sqlite3.Connection):  # pragma: no cover
+    def storage(self) -> Storage:
+        if isinstance(self._storage, sqlite3.Connection):  # pragma: no cover
             raise NotImplementedError(
                 f"self.storage is {self.storage!r}, should be Storage"
             )
-        return self.storage.recent_threshold
+        return self._storage
 
     @property
     def chunk_size(self) -> int:
-        if isinstance(self.storage, sqlite3.Connection):  # pragma: no cover
-            raise NotImplementedError(
-                f"self.storage is {self.storage!r}, should be Storage"
-            )
         return self.storage.chunk_size
 
     # strip_html is not part of the Search interface,
     # but is part of the private API of this implementation.
     strip_html = staticmethod(strip_html)
 
+    @wrap_exceptions(SearchError)
+    def check_dependencies(self) -> None:
+        # Only update() requires these, so we don't check in __init__().
+
+        # last_insert_rowid() / cursor.lastrowid works correctly for FTS5
+        # tables only starting with SQLite 3.18.
+        # https://www.sqlite.org/releaselog/3_18_0.html
+        try:
+            require_version(self.db, (3, 18))
+        except DBError as e:
+            raise SearchError(message=str(e)) from None
+
     @wrap_exceptions(SearchError)
     def enable(self) -> None:
         try:
@@ -302,26 +303,47 @@ class Search:
         # the ON DELETE trigger doesn't fire during REPLACE
         # if recursive_triggers are not enabled (they aren't, as of 1.5).
         #
-        # What I don't understand is why the "INSERT into esss" below succeeds
-        # even when a specific (id, feed) already exists in esss
-        # (it seems to act as INSERT OR REPLACE -- maybe because it already
-        # is in one, and that REPLACE is global?).
-        # Anyway, it seems to do what we need it to do.
+        # Note the entries_search_entries_insert{,_esss_exists} trigers
+        # cannot use OR REPLACE, so we must have one trigger for each case.
 
         self.db.execute(
             """
             CREATE TRIGGER entries_search_entries_insert
             AFTER INSERT ON entries
+
+            WHEN
+                NOT EXISTS (
+                    SELECT *
+                    FROM entries_search_sync_state AS esss
+                    WHERE (esss.id, esss.feed) = (new.id, new.feed)
+                )
+
             BEGIN
-                INSERT INTO entries_search_sync_state (id, feed, es_rowids)
-                VALUES (
-                    new.id,
-                    new.feed,
-                    (
-                        SELECT es_rowids
-                        from entries_search_sync_state
-                        where (id, feed) = (new.id, new.feed)
-                    )
+                INSERT INTO entries_search_sync_state (id, feed)
+                VALUES (new.id, new.feed);
+            END;
+            """
+        )
+        self.db.execute(
+            """
+            CREATE TRIGGER entries_search_entries_insert_esss_exists
+            AFTER INSERT ON entries
+
+            WHEN
+                EXISTS (
+                    SELECT *
+                    FROM entries_search_sync_state AS esss
+                    WHERE (esss.id, esss.feed) = (new.id, new.feed)
+                )
+
+            BEGIN
+                UPDATE entries_search_sync_state
+                SET
+                    to_update = 1,
+                    to_delete = 0
+                WHERE (new.id, new.feed) = (
+                    entries_search_sync_state.id,
+                    entries_search_sync_state.feed
                 );
             END;
             """
@@ -445,6 +467,9 @@ class Search:
         # Private API, may be called from migrations.
         assert self.db.in_transaction
         self.db.execute("DROP TRIGGER IF EXISTS entries_search_entries_insert;")
+        self.db.execute(
+            "DROP TRIGGER IF EXISTS entries_search_entries_insert_esss_exists;"
+        )
         self.db.execute("DROP TRIGGER IF EXISTS entries_search_entries_update;")
         self.db.execute("DROP TRIGGER IF EXISTS entries_search_entries_delete;")
         self.db.execute("DROP TRIGGER IF EXISTS entries_search_feeds_update;")
@@ -465,31 +490,9 @@ class Search:
         return search_table_exists
 
     @wrap_exceptions(SearchError)
+    @wrap_search_exceptions()
     def update(self) -> None:
-        try:
-            return self._update()
-        except sqlite3.OperationalError as e:
-            if 'no such table' in str(e).lower():
-                raise SearchNotEnabledError()
-            raise
-
-    def _update(self) -> None:
-        # If bs4 is not available, we raise an exception here, otherwise
-        # we get just a "user-defined function raised exception" SearchError.
-        if not bs4:
-            raise SearchError(
-                message="could not import search dependencies "
-                "(use the 'search' extra to install them)"
-            ) from bs4_import_error
-
-        # last_insert_rowid() / cursor.lastrowid works correctly for FTS5
-        # tables only starting with SQLite 3.18.
-        # https://www.sqlite.org/releaselog/3_18_0.html
-        try:
-            require_version(self.db, (3, 18))
-        except DBError as e:
-            raise SearchError(message=str(e))
-
+        self.check_dependencies()
         self._delete_from_search()
         self._insert_into_search()
 
@@ -793,7 +796,6 @@ class Search:
             last = None
             if starting_after:
                 if sort == 'recent':
-                    assert isinstance(self.storage, Storage)
                     last = self.storage.get_entry_last(now, sort, starting_after)
                 else:
                     last = self.search_entry_last(query, starting_after)
@@ -822,6 +824,7 @@ class Search:
         yield from rv
 
     @wrap_exceptions(SearchError)
+    @wrap_search_exceptions()
     def search_entry_last(self, query: str, entry: Tuple[str, str]) -> Tuple[Any, ...]:
         feed_url, entry_id = entry
 
@@ -851,7 +854,6 @@ class Search:
         chunk_size: Optional[int] = None,
         last: Optional[_T] = None,
     ) -> Iterable[Tuple[EntrySearchResult, Optional[_T]]]:
-
         sql_query, context = make_search_entries_query(filter_options, sort)
 
         random_mark = ''.join(
@@ -866,7 +868,7 @@ class Search:
             after_mark=after_mark,
             # 255 letters / 4.7 letters per word (average in English)
             snippet_tokens=54,
-            recent_threshold=now - self.recent_threshold,
+            recent_threshold=now - self.storage.recent_threshold,
         )
 
         row_factory = partial(
@@ -875,22 +877,46 @@ class Search:
             after_mark=after_mark,
         )
 
-        with wrap_search_exceptions():
+        with wrap_search_exceptions(query=True):
             yield from paginated_query(
                 self.db, sql_query, context, chunk_size, last, row_factory
             )
 
     @wrap_exceptions(SearchError)
-    @wrap_search_exceptions()
+    @wrap_search_exceptions(query=True)
     def search_entry_counts(
         self,
         query: str,
+        now: datetime,
         filter_options: EntryFilterOptions = EntryFilterOptions(),  # noqa: B008
     ) -> EntrySearchCounts:
-        sql_query, query_context = make_search_entry_counts_query(filter_options)
+        entries_query = (
+            Query()
+            .WITH(
+                (
+                    "search",
+                    """
+                    SELECT _id, _feed
+                    FROM entries_search
+                    WHERE entries_search MATCH :query
+                    GROUP BY _id, _feed
+                    """,
+                )
+            )
+            .SELECT('id', 'feed')
+            .FROM('entries')
+            .JOIN("search ON (id, feed) = (_id, _feed)")
+        )
+        query_context = apply_entry_filter_options(entries_query, filter_options)
+
+        sql_query, new_context = make_entry_counts_query(
+            now, self.storage.entry_counts_average_periods, entries_query
+        )
+        query_context.update(new_context)
+
         context = dict(query=query, **query_context)
         row = exactly_one(self.db.execute(str(sql_query), context))
-        return EntrySearchCounts(*row)
+        return EntrySearchCounts(*row[:4], row[4:7])  # type: ignore[call-arg]
 
 
 def make_search_entries_query(
@@ -1003,42 +1029,3 @@ def entry_search_result_factory(
         MappingProxyType(metadata),
         MappingProxyType(rv_content),
     )
-
-
-def make_search_entry_counts_query(
-    filter_options: EntryFilterOptions,
-) -> Tuple[Query, Dict[str, Any]]:
-    query = (
-        Query()
-        .WITH(
-            (
-                "search",
-                """
-                SELECT _id, _feed
-                FROM entries_search
-                WHERE entries_search MATCH :query
-                GROUP BY _id, _feed
-                """,
-            )
-        )
-        .SELECT(
-            'count(*)',
-            'coalesce(sum(read == 1), 0)',
-            'coalesce(sum(important == 1), 0)',
-            """
-            coalesce(
-                sum(
-                    NOT (
-                        json_array_length(entries.enclosures) IS NULL OR json_array_length(entries.enclosures) = 0
-                    )
-                ), 0
-            )
-            """,
-        )
-        .FROM("entries")
-        .JOIN("search ON (id, feed) = (_id, _feed)")
-    )
-
-    context = apply_entry_filter_options(query, filter_options)
-
-    return query, context
