@@ -36,6 +36,8 @@ from ._utils import chunks
 from ._utils import exactly_one
 from ._utils import join_paginated_iter
 from ._utils import zero_or_one
+from .exceptions import EntryError
+from .exceptions import EntryExistsError
 from .exceptions import EntryMetadataNotFoundError
 from .exceptions import EntryNotFoundError
 from .exceptions import FeedExistsError
@@ -83,6 +85,8 @@ def create_feeds(db: sqlite3.Connection, name: str = 'feeds') -> None:
             link TEXT,
             updated TIMESTAMP,
             author TEXT,
+            subtitle TEXT,
+            version TEXT,
             user_title TEXT,  -- except this one, which comes from reader
             http_etag TEXT,
             http_last_modified TEXT,
@@ -113,7 +117,7 @@ def create_entries(db: sqlite3.Connection, name: str = 'entries') -> None:
             feed TEXT NOT NULL,
             title TEXT,
             link TEXT,
-            updated TIMESTAMP NOT NULL,
+            updated TIMESTAMP,
             author TEXT,
             published TIMESTAMP,
             summary TEXT,
@@ -125,8 +129,12 @@ def create_entries(db: sqlite3.Connection, name: str = 'entries') -> None:
 
             -- reader data
             read INTEGER NOT NULL DEFAULT 0,
+            read_modified TIMESTAMP,
             important INTEGER NOT NULL DEFAULT 0,
+            important_modified TIMESTAMP,
+            added_by TEXT NOT NULL,
             last_updated TIMESTAMP NOT NULL,
+            first_updated TIMESTAMP NOT NULL,
             first_updated_epoch TIMESTAMP NOT NULL,
             feed_order INTEGER NOT NULL,
 
@@ -193,7 +201,7 @@ def create_indexes(db: sqlite3.Connection) -> None:
         """
         CREATE INDEX entries_by_kinda_first_updated ON entries(
             first_updated_epoch,
-            coalesce(published, updated),
+            coalesce(published, updated, first_updated),
             feed,
             last_updated,
             - feed_order,
@@ -204,8 +212,8 @@ def create_indexes(db: sqlite3.Connection) -> None:
     db.execute(
         """
         CREATE INDEX entries_by_kinda_published ON entries (
-            coalesce(published, updated),
-            coalesce(published, updated),
+            coalesce(published, updated, first_updated),
+            coalesce(published, updated, first_updated),
             feed,
             last_updated,
             - feed_order,
@@ -213,24 +221,137 @@ def create_indexes(db: sqlite3.Connection) -> None:
         );
         """
     )
+    # Speed up get_entry_counts(feed=...).
+    db.execute("CREATE INDEX entries_by_feed ON entries (feed);")
+
+
+def update_from_29_to_30(db: sqlite3.Connection) -> None:  # pragma: no cover
+    # for https://github.com/lemon24/reader/issues/251
+    db.execute("CREATE INDEX entries_by_feed ON entries (feed);")
+
+
+def recreate_search_triggers(db: sqlite3.Connection) -> None:  # pragma: no cover
+    from ._search import Search
+
+    search = Search(db)
+    if search.is_enabled():
+        search._drop_triggers()
+        search._create_triggers()
+
+
+def update_from_31_to_32(db: sqlite3.Connection) -> None:  # pragma: no cover
+    # for https://github.com/lemon24/reader/issues/254
+    db.execute("ALTER TABLE entries ADD COLUMN read_modified TIMESTAMP;")
+    db.execute("ALTER TABLE entries ADD COLUMN important_modified TIMESTAMP;")
+
+
+def update_from_32_to_33(db: sqlite3.Connection) -> None:  # pragma: no cover
+    # https://github.com/lemon24/reader/issues/223
+    db.execute("ALTER TABLE feeds ADD COLUMN subtitle TEXT;")
+    db.execute("ALTER TABLE feeds ADD COLUMN version TEXT;")
+    # force all feeds to update regardless of their caching headers
+    db.execute("UPDATE feeds SET stale = 1;")
+
+
+def update_from_33_to_34(db: sqlite3.Connection) -> None:  # pragma: no cover
+    # for https://github.com/lemon24/reader/issues/239
+
+    # Assumes foreign key checks have already been disabled.
+    # https://sqlite.org/lang_altertable.html#otheralter
+
+    # This migration takes care of:
+    #
+    # * first_updated being added not null
+    # * updated becoming null
+    # * re-creating the entries_by_kinda_* indexes to use first_updated
+    # * added_by being added not null
+
+    create_entries(db, 'new_entries')
+    db.execute(
+        """
+        INSERT INTO new_entries (
+            id,
+            feed,
+            title,
+            link,
+            updated,
+            author,
+            published,
+            summary,
+            content,
+            enclosures,
+            original_feed,
+            data_hash,
+            data_hash_changed,
+            read,
+            read_modified,
+            important,
+            important_modified,
+            added_by,
+            last_updated,
+            first_updated,
+            first_updated_epoch,
+            feed_order
+        )
+        SELECT
+            id,
+            feed,
+            title,
+            link,
+            updated,
+            author,
+            published,
+            summary,
+            content,
+            enclosures,
+            original_feed,
+            data_hash,
+            data_hash_changed,
+            read,
+            read_modified,
+            important,
+            important_modified,
+            'feed',
+            last_updated,
+            first_updated_epoch,
+            first_updated_epoch,
+            feed_order
+        FROM entries;
+        """
+    )
+    # IMPORTANT: this drops ALL indexes and triggers ON entries
+    db.execute("DROP TABLE entries;")
+    db.execute("ALTER TABLE new_entries RENAME TO entries;")
+
+    create_indexes(db)
+    recreate_search_triggers(db)
+
+
+MINIMUM_SQLITE_VERSION = (3, 15)
+REQUIRED_SQLITE_COMPILE_OPTIONS = ["ENABLE_JSON1"]
 
 
 def setup_db(db: sqlite3.Connection, wal_enabled: Optional[bool]) -> None:
     return setup_sqlite_db(
         db,
         create=create_db,
-        version=29,
+        version=34,
         migrations={
             # 1-9 removed before 0.1 (last in e4769d8ba77c61ec1fe2fbe99839e1826c17ace7)
             # 10-16 removed before 1.0 (last in 618f158ebc0034eefb724a55a84937d21c93c1a7)
             # 17-28 removed before 2.0 (last in be9c89581ea491d0c9cc95c9d39f073168a2fd02)
+            29: update_from_29_to_30,
+            30: recreate_search_triggers,
+            31: update_from_31_to_32,
+            32: update_from_32_to_33,
+            33: update_from_33_to_34,
         },
         id=APPLICATION_ID,
         # Row value support was added in 3.15.
         # TODO: Remove the Search.update() check once this gets bumped to >=3.18.
-        minimum_sqlite_version=(3, 15),
+        minimum_sqlite_version=MINIMUM_SQLITE_VERSION,
         # We use the JSON1 extension for entries.content.
-        required_sqlite_compile_options=["ENABLE_JSON1"],
+        required_sqlite_compile_options=REQUIRED_SQLITE_COMPILE_OPTIONS,
         wal_enabled=wal_enabled,
     )
 
@@ -258,10 +379,14 @@ def setup_db(db: sqlite3.Connection, wal_enabled: Optional[bool]) -> None:
 
 class Storage:
 
-    # recent_threshold and chunk_size are not part of the Storage interface,
+    # recent_threshold, chunk_size, and entry_counts_average_periods
+    # are not part of the Storage interface,
     # but are part of the private API of this implementation.
     recent_threshold = timedelta(7)
     chunk_size = 2 ** 8
+    # 1, 3, 12 months rounded down to days,
+    # assuming an average of 30.436875 days/month
+    entry_counts_average_periods = (30, 91, 365)
 
     @wrap_exceptions(StorageError)
     def __init__(
@@ -291,7 +416,7 @@ class Storage:
                 message = str(e)
                 if 'no migration' in message:
                     message += "; you may have skipped some required migrations, see https://reader.readthedocs.io/en/latest/changelog.html#removed-migrations-2-0"
-                raise StorageError(message=message)
+                raise StorageError(message=message) from None
 
         self.db: sqlite3.Connection = db
         self.path = path
@@ -329,7 +454,7 @@ class Storage:
             except sqlite3.IntegrityError as e:
                 if "unique constraint failed" not in str(e).lower():  # pragma: no cover
                     raise
-                raise FeedExistsError(url)
+                raise FeedExistsError(url) from None
 
     @wrap_exceptions(StorageError)
     def delete_feed(self, url: str) -> None:
@@ -350,7 +475,7 @@ class Storage:
             except sqlite3.IntegrityError as e:
                 if "unique constraint failed" not in str(e).lower():  # pragma: no cover
                     raise
-                raise FeedExistsError(new)
+                raise FeedExistsError(new) from None
             else:
                 rowcount_exactly_one(cursor, lambda: FeedNotFoundError(old))
 
@@ -361,6 +486,7 @@ class Storage:
                 UPDATE feeds
                 SET
                     updated = NULL,
+                    version = NULL,
                     http_etag = NULL,
                     http_last_modified = NULL,
                     stale = 0,
@@ -508,7 +634,11 @@ class Storage:
                 context = dict(feed_url=feed_url, id=id)
                 row = self.db.execute(
                     """
-                    SELECT updated, data_hash, data_hash_changed
+                    SELECT
+                        updated,
+                        published,
+                        data_hash,
+                        data_hash_changed
                     FROM entries
                     WHERE feed = :feed_url
                         AND id = :id;
@@ -535,6 +665,7 @@ class Storage:
             SELECT
                 entries.id IS NOT NULL,
                 entries.updated,
+                entries.published,
                 entries.data_hash,
                 entries.data_hash_changed
             FROM input
@@ -559,7 +690,9 @@ class Storage:
         iterables = chunks(self.chunk_size, entries) if self.chunk_size else (entries,)
         for iterable in iterables:
 
-            # The reason there are two implementations for this method:
+            # There are two implementations for this method because
+            # the number of parameters in a query varies
+            # (e.g. 999 when compiling from sources, 250k on Ubuntu 18.04);
             # https://github.com/lemon24/reader/issues/109
             iterable = list(iterable)
             try:
@@ -600,30 +733,47 @@ class Storage:
             rowcount_exactly_one(cursor, lambda: FeedNotFoundError(url))
 
     @wrap_exceptions(StorageError)
-    def mark_as_read_unread(self, feed_url: str, entry_id: str, read: bool) -> None:
-        with self.db:
-            cursor = self.db.execute(
-                """
-                UPDATE entries
-                SET read = :read
-                WHERE feed = :feed_url AND id = :entry_id;
-                """,
-                dict(feed_url=feed_url, entry_id=entry_id, read=read),
-            )
-        rowcount_exactly_one(cursor, lambda: EntryNotFoundError(feed_url, entry_id))
-
-    @wrap_exceptions(StorageError)
-    def mark_as_important_unimportant(
-        self, feed_url: str, entry_id: str, important: bool
+    def mark_as_read(
+        self, feed_url: str, entry_id: str, read: bool, modified: Optional[datetime]
     ) -> None:
         with self.db:
             cursor = self.db.execute(
                 """
                 UPDATE entries
-                SET important = :important
+                SET
+                    read = :read,
+                    read_modified = :modified
                 WHERE feed = :feed_url AND id = :entry_id;
                 """,
-                dict(feed_url=feed_url, entry_id=entry_id, important=important),
+                dict(
+                    feed_url=feed_url, entry_id=entry_id, read=read, modified=modified
+                ),
+            )
+        rowcount_exactly_one(cursor, lambda: EntryNotFoundError(feed_url, entry_id))
+
+    @wrap_exceptions(StorageError)
+    def mark_as_important(
+        self,
+        feed_url: str,
+        entry_id: str,
+        important: bool,
+        modified: Optional[datetime],
+    ) -> None:
+        with self.db:
+            cursor = self.db.execute(
+                """
+                UPDATE entries
+                SET
+                    important = :important,
+                    important_modified = :modified
+                WHERE feed = :feed_url AND id = :entry_id;
+                """,
+                dict(
+                    feed_url=feed_url,
+                    entry_id=entry_id,
+                    important=important,
+                    modified=modified,
+                ),
             )
         rowcount_exactly_one(cursor, lambda: EntryNotFoundError(feed_url, entry_id))
 
@@ -672,6 +822,8 @@ class Storage:
                     link = :link,
                     updated = :updated,
                     author = :author,
+                    subtitle = :subtitle,
+                    version = :version,
                     http_etag = :http_etag,
                     http_last_modified = :http_last_modified,
                     data_hash = :data_hash,
@@ -751,9 +903,11 @@ class Storage:
         for iterable in iterables:
             self._add_or_update_entries(iterable)
 
-    def _add_or_update_entries(self, entry_tuples: Iterable[EntryUpdateIntent]) -> None:
+    def _add_or_update_entries(
+        self, entry_tuples: Iterable[EntryUpdateIntent], exclusive: bool = False
+    ) -> None:
         # We need to capture the last value for exception handling
-        # (it's not guaranteed all the entries belong to the same tuple).
+        # (it's not guaranteed all the entries belong to the same feed).
         # FIXME: In this case, is it ok to just fail other feeds too
         # if we have an exception? If no, we should force the entries to
         # belong to a single feed!
@@ -768,8 +922,8 @@ class Storage:
 
             try:
                 self.db.executemany(
-                    """
-                    INSERT OR REPLACE INTO entries (
+                    f"""
+                    INSERT {'OR ABORT' if exclusive else 'OR REPLACE'} INTO entries (
                         id,
                         feed,
                         --
@@ -782,13 +936,17 @@ class Storage:
                         content,
                         enclosures,
                         read,
+                        read_modified,
                         important,
+                        important_modified,
                         last_updated,
+                        first_updated,
                         first_updated_epoch,
                         feed_order,
                         original_feed,
                         data_hash,
-                        data_hash_changed
+                        data_hash_changed,
+                        added_by
                     ) VALUES (
                         :id,
                         :feed_url,
@@ -800,17 +958,32 @@ class Storage:
                         :summary,
                         :content,
                         :enclosures,
-                        (
+                        coalesce((
                             SELECT read
                             FROM entries
                             WHERE id = :id AND feed = :feed_url
-                        ),
+                        ), 0),
                         (
-                        SELECT important
-                        FROM entries
-                        WHERE id = :id AND feed = :feed_url
+                            SELECT read_modified
+                            FROM entries
+                            WHERE id = :id AND feed = :feed_url
+                        ),
+                        coalesce((
+                            SELECT important
+                            FROM entries
+                            WHERE id = :id AND feed = :feed_url
+                        ), 0),
+                        (
+                            SELECT important_modified
+                            FROM entries
+                            WHERE id = :id AND feed = :feed_url
                         ),
                         :last_updated,
+                        coalesce(:first_updated, (
+                            SELECT first_updated
+                            FROM entries
+                            WHERE id = :id AND feed = :feed_url
+                        )),
                         coalesce(:first_updated_epoch, (
                             SELECT first_updated_epoch
                             FROM entries
@@ -819,16 +992,15 @@ class Storage:
                         :feed_order,
                         NULL, -- original_feed
                         :data_hash,
-                        :data_hash_changed
+                        :data_hash_changed,
+                        :added_by
                     );
                     """,
                     make_params(),
                 )
             except sqlite3.IntegrityError as e:
-                if (
-                    "foreign key constraint failed" not in str(e).lower()
-                ):  # pragma: no cover
-                    raise
+                e_msg = str(e).lower()
+
                 feed_url = last_param['feed_url']
                 entry_id = last_param['id']
                 log.debug(
@@ -837,12 +1009,62 @@ class Storage:
                     feed_url,
                     exc_info=True,
                 )
-                raise FeedNotFoundError(feed_url)
+
+                if "foreign key constraint failed" in e_msg:
+                    raise FeedNotFoundError(feed_url) from None
+
+                elif "unique constraint failed: entries.id, entries.feed" in e_msg:
+                    raise EntryExistsError(feed_url, entry_id) from None
+
+                else:  # pragma: no cover
+                    raise
 
     def add_or_update_entry(self, intent: EntryUpdateIntent) -> None:
         # TODO: this method is for testing convenience only, maybe delete it?
         self.add_or_update_entries([intent])
 
+    @wrap_exceptions(StorageError)
+    def add_entry(self, intent: EntryUpdateIntent) -> None:
+        # TODO: unify with the or_update variants
+        self._add_or_update_entries([intent], exclusive=True)
+
+    @wrap_exceptions(StorageError)
+    def delete_entries(
+        self, entries: Iterable[Tuple[str, str]], *, added_by: Optional[str] = None
+    ) -> None:
+        # This must be atomic (unlike add_or_update_entries()); hence, no paging.
+        # We'll deal with locking issues only if they start appearing
+        # (hopefully, there are both fewer entries to be deleted and
+        # this takes less time per entry).
+
+        # I don't know how to check *which* entry did not exit
+        # with executemany(), so we use execute().
+        # TODO: Maybe add_or_update_entries() doesn't need executemany() either.
+
+        delete_query = "DELETE FROM entries WHERE feed = :feed AND id = :id"
+        added_by_query = "SELECT added_by FROM entries WHERE feed = :feed AND id = :id"
+
+        with self.db:
+            cursor = self.db.cursor()
+
+            for feed_url, entry_id in entries:
+                context = dict(feed=feed_url, id=entry_id)
+
+                if added_by is not None:
+                    row = cursor.execute(added_by_query, context).fetchone()
+                    if row:
+                        if row[0] != added_by:
+                            raise EntryError(
+                                feed_url,
+                                entry_id,
+                                f"entry must be added by {added_by!r}, got {row[0]!r}",
+                            )
+
+                cursor.execute(delete_query, context)
+                rowcount_exactly_one(
+                    cursor, lambda: EntryNotFoundError(feed_url, entry_id)
+                )
+
     def get_entries(
         self,
         now: datetime,
@@ -923,32 +1145,21 @@ class Storage:
     @wrap_exceptions(StorageError)
     def get_entry_counts(
         self,
+        now: datetime,
         filter_options: EntryFilterOptions = EntryFilterOptions(),  # noqa: B008
     ) -> EntryCounts:
-        query = (
-            Query()
-            .SELECT(
-                'count(*)',
-                'coalesce(sum(read == 1), 0)',
-                'coalesce(sum(important == 1), 0)',
-                """
-                coalesce(
-                    sum(
-                        NOT (
-                            json_array_length(entries.enclosures) IS NULL OR json_array_length(entries.enclosures) = 0
-                        )
-                    ), 0
-                )
-                """,
-            )
-            .FROM("entries")
-        )
 
-        context = apply_entry_filter_options(query, filter_options)
+        entries_query = Query().SELECT('id', 'feed').FROM('entries')
+        context = apply_entry_filter_options(entries_query, filter_options)
+
+        query, new_context = make_entry_counts_query(
+            now, self.entry_counts_average_periods, entries_query
+        )
+        context.update(new_context)
 
         row = exactly_one(self.db.execute(str(query), context))
 
-        return EntryCounts(*row)
+        return EntryCounts(*row[:4], row[4:7])  # type: ignore[call-arg]
 
     def iter_metadata(
         self,
@@ -1013,7 +1224,7 @@ class Storage:
                 foreign_key_error = "foreign key constraint failed" in str(e).lower()
                 if not foreign_key_error:  # pragma: no cover
                     raise
-                raise info.not_found_exc(*object_id)
+                raise info.not_found_exc(*object_id) from None
 
     @wrap_exceptions(StorageError)
     def delete_metadata(self, object_id: Tuple[str, ...], key: str) -> None:
@@ -1055,7 +1266,7 @@ class Storage:
                 self.db.execute(query, params)
             except sqlite3.IntegrityError as e:
                 if "foreign key constraint failed" in str(e).lower():
-                    raise info.not_found_exc(*object_id)
+                    raise info.not_found_exc(*object_id) from None
                 # tag exists is a no-op; it looks like:
                 # "UNIQUE constraint failed: feed_tags.feed, feed_tags.tag"
 
@@ -1128,6 +1339,8 @@ def make_get_feeds_query(
             'title',
             'link',
             'author',
+            'subtitle',
+            'version',
             'user_title',
             'added',
             'last_updated',
@@ -1157,10 +1370,10 @@ def make_get_feeds_query(
 
 def feed_factory(t: Tuple[Any, ...]) -> Feed:
     return Feed._make(
-        t[:8]
+        t[:10]
         + (
-            ExceptionInfo(**json.loads(t[8])) if t[8] else None,
-            t[9] == 1,
+            ExceptionInfo(**json.loads(t[10])) if t[10] else None,
+            t[11] == 1,
         )
     )
 
@@ -1200,6 +1413,8 @@ def make_get_entries_query(
             feeds.title
             feeds.link
             feeds.author
+            feeds.subtitle
+            feeds.version
             feeds.user_title
             feeds.added
             feeds.last_updated
@@ -1215,7 +1430,11 @@ def make_get_entries_query(
             entries.content
             entries.enclosures
             entries.read
+            entries.read_modified
             entries.important
+            entries.important_modified
+            entries.first_updated
+            entries.added_by
             entries.last_updated
             entries.original_feed
             """.split()
@@ -1241,14 +1460,18 @@ def make_get_entries_query(
 
 
 def entry_factory(t: Tuple[Any, ...]) -> Entry:
-    feed = feed_factory(t[0:10])
-    entry = t[10:17] + (
-        tuple(Content(**d) for d in json.loads(t[17])) if t[17] else (),
-        tuple(Enclosure(**d) for d in json.loads(t[18])) if t[18] else (),
-        t[19] == 1,
-        t[20] == 1,
-        t[21],
-        t[22] or feed.url,
+    feed = feed_factory(t[0:12])
+    entry = t[12:19] + (
+        tuple(Content(**d) for d in json.loads(t[19])) if t[19] else (),
+        tuple(Enclosure(**d) for d in json.loads(t[20])) if t[20] else (),
+        t[21] == 1,
+        t[22],
+        t[23] == 1,
+        t[24],
+        t[25],
+        t[26],
+        t[27],
+        t[28] or feed.url,
         feed,
     )
     return Entry._make(entry)
@@ -1349,15 +1572,15 @@ def make_recent_last_select(id_prefix: str = 'entries.') -> Sequence[Any]:
             coalesce (
                 CASE
                 WHEN
-                    coalesce(entries.published, entries.updated)
+                    coalesce(entries.published, entries.updated, entries.first_updated)
                         >= :recent_threshold
                     THEN entries.first_updated_epoch
                 END,
-                entries.published, entries.updated
+                entries.published, entries.updated, entries.first_updated
             )
             """,
         ),
-        ('kinda_published', 'coalesce(published, updated)'),
+        ('kinda_published', 'coalesce(published, updated, first_updated)'),
         f'{id_prefix}feed',
         'last_updated',
         ('negative_feed_order', '- feed_order'),
@@ -1381,7 +1604,7 @@ def apply_recent(
             kinda_first_updated = 'first_updated_epoch'
             sign = '>='
         else:
-            kinda_first_updated = 'coalesce(published, updated)'
+            kinda_first_updated = 'coalesce(published, updated, first_updated)'
             sign = '<'
 
         return f"""
@@ -1390,7 +1613,7 @@ def apply_recent(
                 id,
                 last_updated,
                 {kinda_first_updated} as kinda_first_updated,
-                coalesce(published, updated) as kinda_published,
+                coalesce(published, updated, first_updated) as kinda_published,
                 - feed_order as negative_feed_order
             FROM entries
             WHERE kinda_published {sign} :recent_threshold
@@ -1447,3 +1670,58 @@ def apply_random(query: Query) -> None:
     # can benefit from future optimizations.
     #
     query.ORDER_BY("random()")
+
+
+def make_entry_counts_query(
+    now: datetime,
+    average_periods: Tuple[float, ...],
+    entries_query: Query,
+) -> Tuple[Query, Dict[str, Any]]:
+    query = (
+        Query()
+        .WITH(('entries_filtered', str(entries_query)))
+        .SELECT(
+            'count(*)',
+            'coalesce(sum(read == 1), 0)',
+            'coalesce(sum(important == 1), 0)',
+            """
+            coalesce(
+                sum(
+                    NOT (
+                        json_array_length(entries.enclosures) IS NULL OR json_array_length(entries.enclosures) = 0
+                    )
+                ), 0
+            )
+            """,
+        )
+        .FROM("entries_filtered")
+        .JOIN("entries USING (id, feed)")
+    )
+
+    # one CTE / period + HAVING in the CTE is a tiny bit faster than
+    # one CTE + WHERE in the SELECT
+
+    context: Dict[str, Any] = dict(now=now)
+
+    for period_i, period_days in enumerate(average_periods):
+        # TODO: when we get first_updated, use it instead of first_updated_epoch
+
+        days_param = f'kfu_{period_i}_days'
+        context[days_param] = float(period_days)
+
+        start_param = f'kfu_{period_i}_start'
+        context[start_param] = now - timedelta(days=period_days)
+
+        kfu_query = (
+            Query()
+            .SELECT('coalesce(published, updated, first_updated_epoch) AS kfu')
+            .FROM('entries_filtered')
+            .JOIN("entries USING (id, feed)")
+            .GROUP_BY('published, updated, first_updated_epoch, feed')
+            .HAVING(f"kfu BETWEEN :{start_param} AND :now")
+        )
+
+        query.WITH((f'kfu_{period_i}', str(kfu_query)))
+        query.SELECT(f"(SELECT count(*) / :{days_param} FROM kfu_{period_i})")
+
+    return query, context
