@@ -2,8 +2,8 @@ from dataclasses import dataclass
 from datetime import datetime
 from datetime import timezone
 from types import MappingProxyType
+from types import SimpleNamespace
 from typing import Any
-from typing import Generic
 from typing import Iterable
 from typing import Mapping
 from typing import NamedTuple
@@ -22,6 +22,7 @@ from .types import _namedtuple_compat
 from .types import Content
 from .types import Enclosure
 from .types import Entry
+from .types import EntryAddedBy
 from .types import EntryInput
 from .types import ExceptionInfo
 from .types import Feed
@@ -35,6 +36,9 @@ from .types import TagFilterInput
 # https://github.com/lemon24/reader/issues/159#issuecomment-612512033
 
 
+_T = TypeVar('_T')
+
+
 @dataclass(frozen=True)
 class FeedData(_namedtuple_compat):
 
@@ -49,6 +53,8 @@ class FeedData(_namedtuple_compat):
     title: Optional[str] = None
     link: Optional[str] = None
     author: Optional[str] = None
+    subtitle: Optional[str] = None
+    version: Optional[str] = None
 
     def as_feed(self, **kwargs: object) -> Feed:
         """For testing."""
@@ -68,47 +74,34 @@ class FeedData(_namedtuple_compat):
         return get_hash(self)
 
 
-_UpdatedType = TypeVar('_UpdatedType', datetime, Optional[datetime])
-
-
 @dataclass(frozen=True)
-class EntryData(Generic[_UpdatedType], _namedtuple_compat):
+class EntryData(_namedtuple_compat):
 
     """Entry data that comes from the feed.
 
     Attributes are a subset of those of Entry.
 
-    The natural thing to use would have been generics, but pleasing Python,
-    mypy and Sphinx all at the same time is not possible at the moment,
-    and the workarounds are just as bad or worse.
+    ---
 
-    We should be able to use generics once/if this is resolved:
-    https://github.com/sphinx-doc/sphinx/issues/7450
+    This is not generic anymore, as of 2.5, and will likely never be.
 
-    ...however, it may be better to just have entry be a separate
-    plain dataclass -- help(Entry) works weird with concrete generics.
+    TODO: Make Entry a subclass of EntryData, make Feed a subclass of FeedData.
 
-    We can't subclass Entry because the attribute types become less specific.
+    It may still not be possible to use it as a subclass, though, because:
 
-    We can't use a subclass for the common attributes because it confuses
-    Sphinx: https://github.com/sphinx-doc/sphinx/issues/741
-
-    An implementation using generics is available here:
-    https://github.com/lemon24/reader/blob/62eb72563b94d78d8860519424103e3c3c1c013d/src/reader/core/types.py#L78-L241
+    * help(Entry) may not work
+    * Sphinx/autodoc may not work: https://github.com/sphinx-doc/sphinx/issues/741 (closed)
+    * as_entry(), hash() must not be inherited
 
     """
 
     #: The feed URL.
     feed_url: str
 
-    # WARNING: When changing attributes, keep Entry and EntryData in sync.
+    # WARNING: When changing attributes, keep Entry, EntryData, and entry_data_from_obj in sync.
 
     id: str
-
-    # Entries returned by the parser have .updated Optional[datetime];
-    # entries sent to the storage always have .updatd set (not optional).
-    updated: _UpdatedType
-
+    updated: Optional[datetime] = None
     title: Optional[str] = None
     link: Optional[str] = None
     author: Optional[str] = None
@@ -124,6 +117,7 @@ class EntryData(Generic[_UpdatedType], _namedtuple_compat):
         attrs.pop('hash', None)
         attrs.update(kwargs)
         attrs.setdefault('original_feed_url', feed_url)
+        attrs.setdefault('added_by', 'feed')
         return Entry(**attrs)
 
     @property
@@ -137,11 +131,82 @@ class EntryData(Generic[_UpdatedType], _namedtuple_compat):
         return get_hash(self)
 
 
+def entry_data_from_obj(obj: object) -> EntryData:
+    """Union[EntryDataLikeProtocol, EntryDataTypedDict] -> EntryData
+
+    Naive datetimes are normalized by passing them to
+    :meth:`~datetime.datetime.astimezone`.
+
+    """
+    if isinstance(obj, Mapping):
+        obj = SimpleNamespace(**obj)
+    return EntryData(
+        feed_url=_getattr(obj, 'feed_url', str),
+        id=_getattr(obj, 'id', str),
+        updated=_getattr_optional_datetime(obj, 'updated'),
+        title=_getattr_optional(obj, 'title', str),
+        link=_getattr_optional(obj, 'link', str),
+        author=_getattr_optional(obj, 'author', str),
+        published=_getattr_optional_datetime(obj, 'published'),
+        summary=_getattr_optional(obj, 'summary', str),
+        content=tuple(content_from_obj(o) for o in getattr(obj, 'content', ())),
+        enclosures=tuple(enclosure_from_obj(o) for o in getattr(obj, 'enclosures', ())),
+    )
+
+
+def content_from_obj(obj: object) -> Content:
+    if isinstance(obj, Mapping):
+        obj = SimpleNamespace(**obj)
+    return Content(
+        value=_getattr(obj, 'value', str),
+        type=_getattr_optional(obj, 'type', str),
+        language=_getattr_optional(obj, 'language', str),
+    )
+
+
+def enclosure_from_obj(obj: object) -> Enclosure:
+    if isinstance(obj, Mapping):
+        obj = SimpleNamespace(**obj)
+    return Enclosure(
+        href=_getattr(obj, 'href', str),
+        type=_getattr_optional(obj, 'type', str),
+        length=_getattr_optional(obj, 'length', int),
+    )
+
+
+def _getattr(obj: object, name: str, type: Type[_T]) -> _T:
+    # will raise AttributeError implicitly
+    value = getattr(obj, name)
+    if not isinstance(value, type):
+        raise TypeError(
+            f"bad type for {name}; expected {type.__name__} instance, got {value!r}"
+        )
+    return value
+
+
+def _getattr_optional(obj: object, name: str, type: Type[_T]) -> Optional[_T]:
+    value = getattr(obj, name, None)
+    if value is None:
+        return value
+    if not isinstance(value, type):
+        raise TypeError(
+            f"bad type for {name}; expected {type.__name__} instance, got {value!r}"
+        )
+    return value
+
+
+def _getattr_optional_datetime(obj: object, name: str) -> Optional[datetime]:
+    value = _getattr_optional(obj, name, datetime)
+    if value is None:
+        return value
+    return value.astimezone(timezone.utc).replace(tzinfo=None)
+
+
 class ParsedFeed(NamedTuple):
 
     feed: FeedData
     # TODO: wrap entries in iter(entries) to ensure stuff doesn't rely on it being a list
-    entries: Iterable[EntryData[Optional[datetime]]]
+    entries: Iterable[EntryData]
     http_etag: Optional[str] = None
     http_last_modified: Optional[str] = None
 
@@ -177,7 +242,10 @@ class EntryForUpdate(NamedTuple):
     """Update-relevant information about an existing entry, from Storage."""
 
     #: The date the entry was last updated, according to the entry.
-    updated: datetime
+    updated: Optional[datetime]
+
+    #: The date the entry was published, according to the entry.
+    published: Optional[datetime]
 
     #: The hash of the corresponding EntryData.
     hash: Optional[bytes]
@@ -211,22 +279,29 @@ class EntryUpdateIntent(NamedTuple):
     """Data to be passed to Storage when updating a feed."""
 
     #: The entry.
-    entry: EntryData[datetime]
+    entry: EntryData
 
     #: The time at the start of updating this feed (start of update_feed
     #: in update_feed, the start of each feed update in update_feeds).
     last_updated: datetime
 
+    #: First last_updated (sets Entry.added).
+    #: None if the entry already exists.
+    first_updated: Optional[datetime]
+
     #: The time at the start of updating this batch of feeds (start of
     #: update_feed in update_feed, start of update_feeds in update_feeds);
     #: None if the entry already exists.
     first_updated_epoch: Optional[datetime]
 
     #: The index of the entry in the feed (zero-based).
-    feed_order: int
+    feed_order: int = 0
 
     #: Same as EntryForUpdate.hash_changed.
-    hash_changed: Optional[int]
+    hash_changed: Optional[int] = 0
+
+    #: Same as Entry.source.
+    added_by: EntryAddedBy = 'feed'
 
     @property
     def new(self) -> bool:
@@ -394,16 +469,16 @@ DEFAULT_RESERVED_NAME_SCHEME = MappingProxyType(
 )
 
 
-_T = TypeVar('_T', bound=_namedtuple_compat)
+_NT = TypeVar('_NT', bound=_namedtuple_compat)
 
 
 def fix_datetime_tzinfo(
-    obj: _T,
+    obj: _NT,
     *names: str,
     _old: Union[None, timezone, bool] = None,
     _new: Union[None, timezone] = timezone.utc,
     **kwargs: Any,
-) -> _T:
+) -> _NT:
     """For specific optional datetime attributes of an object,
     and set their tzinfo to `_new`.
 
