@@ -5,12 +5,13 @@ import numbers
 import warnings
 from contextlib import nullcontext
 from datetime import datetime
+from datetime import timezone
 from types import MappingProxyType
 from typing import Any
 from typing import Callable
-from typing import Collection
 from typing import Iterable
 from typing import Iterator
+from typing import List
 from typing import Mapping
 from typing import MutableSequence
 from typing import Optional
@@ -19,6 +20,8 @@ from typing import Tuple
 from typing import TypeVar
 from typing import Union
 
+from typing_extensions import Literal
+
 import reader._updater
 from ._parser import default_parser
 from ._parser import Parser
@@ -27,6 +30,7 @@ from ._requests_utils import TimeoutType
 from ._search import Search
 from ._storage import Storage
 from ._types import DEFAULT_RESERVED_NAME_SCHEME
+from ._types import entry_data_from_obj
 from ._types import EntryData
 from ._types import EntryFilterOptions
 from ._types import EntryUpdateIntent
@@ -43,6 +47,7 @@ from .exceptions import FeedMetadataNotFoundError
 from .exceptions import FeedNotFoundError
 from .exceptions import InvalidPluginError
 from .exceptions import ParseError
+from .exceptions import SearchNotEnabledError
 from .plugins import _DEFAULT_PLUGINS
 from .plugins import _PLUGINS
 from .types import _entry_argument
@@ -74,10 +79,8 @@ _T = TypeVar('_T')
 _U = TypeVar('_U')
 
 ReaderPluginType = Callable[['Reader'], None]
-AfterEntryUpdateHook = Callable[
-    ['Reader', EntryData[datetime], EntryUpdateStatus], None
-]
-_PostFeedUpdatePluginType = Callable[['Reader', str], None]
+AfterEntryUpdateHook = Callable[['Reader', EntryData, EntryUpdateStatus], None]
+AfterFeedUpdateHook = Callable[['Reader', str], None]
 
 
 def make_reader(
@@ -87,6 +90,7 @@ def make_reader(
     plugins: Iterable[Union[str, ReaderPluginType]] = _DEFAULT_PLUGINS,
     session_timeout: TimeoutType = SESSION_TIMEOUT,
     reserved_name_scheme: Mapping[str, str] = DEFAULT_RESERVED_NAME_SCHEME,
+    search_enabled: Union[bool, None, Literal['auto']] = 'auto',
     _storage: Optional[Storage] = None,
     _storage_factory: Any = None,
 ) -> 'Reader':
@@ -161,20 +165,29 @@ def make_reader(
             as a float, or a (connect timeout, read timeout) tuple.
             Passed to the underlying `Requests session`_.
 
-        reserved_name_scheme (dict(str, str) or None):
+        reserved_name_scheme (dict(str, str)):
             Value for :attr:`~Reader.reserved_name_scheme`.
             The prefixes default to ``.reader.``/``.plugin.``,
             and the separator to ``.``
 
+        search_enabled (bool or None or ``'auto'``):
+            Whether to enable search. One of
+            ``'auto'`` (enable on the first
+            :meth:`~Reader.update_search` call; default),
+            :const:`True` (enable),
+            :const:`False` (disable),
+            :const:`None` (do nothing).
+
     .. _Requests session: https://requests.readthedocs.io/en/master/user/advanced/#timeouts
 
     Returns:
         Reader: The reader.
 
     Raises:
-        StorageError
+        StorageError: An error occurred while connecting to storage.
+        SearchError: An error occurred while enabling/disabling search.
         InvalidPluginError: An invalid plugin name was passed to ``plugins``.
-        ReaderError: An ambiguous exception ocurred while creating the reader.
+        ReaderError: An ambiguous exception occurred while creating the reader.
 
     .. versionadded:: 1.6
         The ``feed_root`` keyword argument.
@@ -189,14 +202,44 @@ def make_reader(
         raises :exc:`InvalidPluginError`, a :exc:`ValueError` subclass.
 
     .. versionadded:: 1.17
-        The ``reserved_name_scheme`` argument.
+        The ``reserved_name_scheme`` keyword argument.
 
     .. versionchanged:: 2.0
         ``feed_root`` now defaults to ``None`` (don't open local feeds)
         instead of ``''`` (full filesystem access).
 
+    .. versionadded:: 2.4
+        The ``search_enabled`` keyword argument.
+
+    .. versionchanged:: 2.4
+        Enable search on the first :meth:`~Reader.update_search` call.
+        To get the previous behavior (leave search as-is),
+        use ``search_enabled=None``.
+
     """
 
+    # Do as much work as possible before creating the storage.
+
+    if search_enabled not in ('auto', True, False, None):
+        raise ValueError("search_enabled should be one of ('auto', True, False, None)")
+
+    parser = default_parser(feed_root, session_timeout=session_timeout)
+
+    try:
+        name_scheme = NameScheme.from_value(reserved_name_scheme)
+    except Exception as e:
+        raise ValueError(f"invalid reserved name scheme: {reserved_name_scheme}") from e
+
+    plugin_funcs: List[ReaderPluginType] = []
+    for plugin in plugins:
+        if isinstance(plugin, str):
+            if plugin not in _PLUGINS:
+                raise InvalidPluginError(f"no such built-in plugin: {plugin!r}")
+            plugin_func = _PLUGINS[plugin]
+        else:
+            plugin_func = plugin
+        plugin_funcs.append(plugin_func)
+
     # If we ever need to change the signature of make_reader(),
     # or support additional storage/search implementations,
     # we'll need to do the wiring differently.
@@ -206,31 +249,32 @@ def make_reader(
 
     storage = _storage or Storage(url, factory=_storage_factory)
 
-    # For now, we're using a storage-bound search provider.
-    search = Search(storage)
+    try:
+        # For now, we're using a storage-bound search provider.
+        search = Search(storage)
 
-    parser = default_parser(feed_root, session_timeout=session_timeout)
+        if search_enabled is True:
+            search.check_dependencies()
+            search.enable()
+        elif search_enabled is False:
+            search.disable()
 
-    reader = Reader(
-        storage, search, parser, reserved_name_scheme, _called_directly=False
-    )
+        reader = Reader(
+            storage,
+            search,
+            parser,
+            name_scheme,
+            _enable_search=(search_enabled == 'auto'),
+            _called_directly=False,
+        )
 
-    for plugin in plugins:
-        if isinstance(plugin, str):
-            if plugin not in _PLUGINS:
-                raise InvalidPluginError(f"no such built-in plugin: {plugin!r}")
+        # TODO: (maybe) wrap exceptions raised here in a custom exception
+        for plugin_func in plugin_funcs:
+            plugin_func(reader)
 
-            plugin_func = _PLUGINS[plugin]
-        else:
-            plugin_func = plugin
-
-        try:
-            plugin_func(reader)  # type: ignore
-        except Exception:  # pragma: no cover
-            # TODO: this whole branch is not tested
-            reader.close()
-            # TODO: this should raise a custom exception (but can't because of backwards compatibility)
-            raise
+    except BaseException:
+        storage.close()
+        raise
 
     return reader
 
@@ -278,20 +322,19 @@ class Reader:
         _storage: Storage,
         _search: Search,
         _parser: Parser,
-        _reserved_name_scheme: Mapping[str, str],
+        _reserved_name_scheme: NameScheme,
+        _enable_search: bool = False,
         _called_directly: bool = True,
     ):
         self._storage = _storage
         self._search = _search
         self._parser = _parser
 
-        try:
-            self.reserved_name_scheme = _reserved_name_scheme
-        except AttributeError as e:
-            raise ValueError(str(e)) from (e.__cause__ or e)
+        self._reserved_name_scheme = _reserved_name_scheme
+
+        self._enable_search = _enable_search
 
         self._updater = reader._updater
-        self._post_feed_update_plugins: Collection[_PostFeedUpdatePluginType] = []
 
         #: List of functions called for each updated entry
         #: after the feed was updated.
@@ -316,6 +359,20 @@ class Reader:
         #:
         self.after_entry_update_hooks: MutableSequence[AfterEntryUpdateHook] = []
 
+        #: List of functions called for each updated feed
+        #: after the feed was updated.
+        #:
+        #: Each function is called with:
+        #:
+        #: * `reader` – the :class:`Reader` instance
+        #: * `feed` – the :class:`str` feed URL
+        #:
+        #: Each function should return :const:`None`.
+        #:
+        #: .. versionadded:: 2.2
+        #:
+        self.after_feed_update_hooks: MutableSequence[AfterFeedUpdateHook] = []
+
         if _called_directly:
             warnings.warn(
                 "Reader objects should be created using make_reader(); the Reader "
@@ -336,20 +393,35 @@ class Reader:
         """
         self._storage.close()
 
-    def add_feed(self, feed: FeedInput) -> None:
+    def add_feed(self, feed: FeedInput, *, allow_invalid_url: bool = False) -> None:
         """Add a new feed.
 
         Feed updates are enabled by default.
 
         Args:
             feed (str or Feed): The feed URL.
+            allow_invalid_url (bool):
+                Add feed even if the current Reader configuration
+                does not know how to handle the feed URL
+                (and updates for it would fail).
 
         Raises:
             FeedExistsError
             StorageError
+            InvalidFeedURLError: If ``feed`` is invalid and ``allow_invalid_url`` is false.
+
+        .. versionadded:: 2.5
+            The ``allow_invalid_url`` keyword argument.
+
+        .. versionchanged:: 2.5
+            Validate the new feed URL.
+            To get the previous behavior (no validation),
+            use ``allow_invalid_url=True``.
 
         """
         url = _feed_argument(feed)
+        if not allow_invalid_url:
+            self._parser.validate_url(url)
         now = self._now()
         self._storage.add_feed(url, now)
 
@@ -370,15 +442,19 @@ class Reader:
         url = _feed_argument(feed)
         self._storage.delete_feed(url)
 
-    def change_feed_url(self, old: FeedInput, new: FeedInput) -> None:
+    def change_feed_url(
+        self, old: FeedInput, new: FeedInput, *, allow_invalid_url: bool = False
+    ) -> None:
         """Change the URL of a feed.
 
         User-defined feed attributes are preserved:
         :attr:`~Feed.added`, :attr:`~Feed.user_title`.
         Feed-defined feed attributes are also preserved,
         at least until the next update:
-        :attr:`~Feed.title`, :attr:`~Feed.link`, :attr:`~Feed.author`
-        (except :attr:`~Feed.updated`, which gets set to None).
+        :attr:`~Feed.title`, :attr:`~Feed.link`, :attr:`~Feed.author`,
+        :attr:`~Feed.subtitle`
+        (except :attr:`~Feed.updated` and :attr:`~Feed.version`,
+        which get set to None).
         All other feed attributes are set to their default values.
 
         The entries, tags and metadata are preserved.
@@ -386,16 +462,33 @@ class Reader:
         Args:
             old (str or Feed): The old feed; must exist.
             new (str or Feed): The new feed; must not exist.
+            allow_invalid_url (bool):
+                Change feed URL even if the current Reader configuration
+                does not know how to handle the new feed URL
+                (and updates for it would fail).
 
         Raises:
             FeedNotFoundError: If ``old`` does not exist.
             FeedExistsError: If ``new`` already exists.
             StorageError
+            InvalidFeedURLError: If ``new`` is invalid and ``allow_invalid_url`` is false.
 
         .. versionadded:: 1.8
 
+        .. versionadded:: 2.5
+            The ``allow_invalid_url`` keyword argument.
+
+        .. versionchanged:: 2.5
+            Validate the new feed URL.
+            To get the previous behavior (no validation),
+            use ``allow_invalid_url=True``.
+
         """
-        self._storage.change_feed_url(_feed_argument(old), _feed_argument(new))
+        old_str = _feed_argument(old)
+        new_str = _feed_argument(new)
+        if not allow_invalid_url:
+            self._parser.validate_url(new_str)
+        self._storage.change_feed_url(old_str, new_str)
 
     def get_feeds(
         self,
@@ -821,6 +914,8 @@ class Reader:
         # for the second of two new feeds updated in the same update_feeds()
         # call, first_updated_epoch != last_updated.
         #
+        # Update: However, added == last_updated for the first update.
+        #
         global_now = self._now()
 
         # Excluding the special exception handling,
@@ -912,9 +1007,6 @@ class Reader:
         # if feed_for_update.url != parsed_feed.feed.url, the feed was redirected.
         # TODO: Maybe handle redirects somehow else (e.g. change URL if permanent).
 
-        for feed_plugin in self._post_feed_update_plugins:
-            feed_plugin(self, url)
-
         new_count = 0
         updated_count = 0
         for entry in entries_to_update:
@@ -927,6 +1019,9 @@ class Reader:
             for entry_hook in self.after_entry_update_hooks:
                 entry_hook(self, entry.entry, entry_status)
 
+        for feed_hook in self.after_feed_update_hooks:
+            feed_hook(self, url)
+
         return new_count, updated_count
 
     def get_entries(
@@ -1036,7 +1131,10 @@ class Reader:
                 rv_entry,
                 'updated',
                 'published',
+                'added',
                 'last_updated',
+                'read_modified',
+                'important_modified',
                 feed=fix_datetime_tzinfo(
                     rv_entry.feed, 'updated', 'added', 'last_updated'
                 ),
@@ -1116,11 +1214,51 @@ class Reader:
         filter_options = EntryFilterOptions.from_args(
             feed, entry, read, important, has_enclosures, feed_tags
         )
-        return self._storage.get_entry_counts(filter_options)
+        now = self._now()
+        return self._storage.get_entry_counts(now, filter_options)
+
+    def set_entry_read(
+        self,
+        entry: EntryInput,
+        read: bool,
+        modified: Union[MissingType, None, datetime] = MISSING,
+    ) -> None:
+        """Mark an entry as read or unread,
+        possibly with a custom timestamp.
+
+        Args:
+            entry (tuple(str, str) or Entry): (feed URL, entry id) tuple.
+            read (bool): Mark the entry as read if true (default),
+                and as unread otherwise.
+            modified (datetime or None):
+                Set :attr:`~Entry.read_modified` to this.
+                Naive datetimes are normalized by passing them to
+                :meth:`~datetime.datetime.astimezone`.
+                Defaults to the current time.
+
+        Raises:
+            EntryNotFoundError
+            StorageError
+
+        .. versionadded:: 2.2
+
+        """
+        modified_naive: Optional[datetime]
+        if isinstance(modified, MissingType):
+            modified_naive = self._now()
+        elif modified is None:
+            modified_naive = None
+        else:
+            modified_naive = modified.astimezone(timezone.utc).replace(tzinfo=None)
+
+        feed_url, entry_id = _entry_argument(entry)
+        self._storage.mark_as_read(feed_url, entry_id, bool(read), modified_naive)
 
     def mark_entry_as_read(self, entry: EntryInput) -> None:
         """Mark an entry as read.
 
+        Alias for ``set_entry_read(entry, True)``.
+
         Args:
             entry (tuple(str, str) or Entry): (feed URL, entry id) tuple.
 
@@ -1132,12 +1270,13 @@ class Reader:
             Renamed from :meth:`mark_as_read`.
 
         """
-        feed_url, entry_id = _entry_argument(entry)
-        self._storage.mark_as_read_unread(feed_url, entry_id, True)
+        self.set_entry_read(entry, True)
 
     def mark_entry_as_unread(self, entry: EntryInput) -> None:
         """Mark an entry as unread.
 
+        Alias for ``set_entry_read(entry, False)``.
+
         Args:
             entry (tuple(str, str) or Entry): (feed URL, entry id) tuple.
 
@@ -1149,12 +1288,52 @@ class Reader:
             Renamed from :meth:`mark_as_unread`.
 
         """
+        return self.set_entry_read(entry, False)
+
+    def set_entry_important(
+        self,
+        entry: EntryInput,
+        important: bool,
+        modified: Union[MissingType, None, datetime] = MISSING,
+    ) -> None:
+        """Mark an entry as important or unimportant,
+        possibly with a custom timestamp.
+
+        Args:
+            entry (tuple(str, str) or Entry): (feed URL, entry id) tuple.
+            important (bool): Mark the entry as important if true (default),
+                and as unimportant otherwise.
+            modified (datetime or None):
+                Set :attr:`~Entry.important_modified` to this.
+                Naive datetimes are normalized by passing them to
+                :meth:`~datetime.datetime.astimezone`.
+                Defaults to the current time.
+
+        Raises:
+            EntryNotFoundError
+            StorageError
+
+        .. versionadded:: 2.2
+
+        """
+        modified_naive: Optional[datetime]
+        if isinstance(modified, MissingType):
+            modified_naive = self._now()
+        elif modified is None:
+            modified_naive = None
+        else:
+            modified_naive = modified.astimezone(timezone.utc).replace(tzinfo=None)
+
         feed_url, entry_id = _entry_argument(entry)
-        self._storage.mark_as_read_unread(feed_url, entry_id, False)
+        self._storage.mark_as_important(
+            feed_url, entry_id, bool(important), modified_naive
+        )
 
     def mark_entry_as_important(self, entry: EntryInput) -> None:
         """Mark an entry as important.
 
+        Alias for ``set_entry_important(entry, True)``.
+
         Args:
             entry (tuple(str, str) or Entry): (feed URL, entry id) tuple.
 
@@ -1166,12 +1345,13 @@ class Reader:
             Renamed from :meth:`mark_as_important`.
 
         """
-        feed_url, entry_id = _entry_argument(entry)
-        self._storage.mark_as_important_unimportant(feed_url, entry_id, True)
+        self.set_entry_important(entry, True)
 
     def mark_entry_as_unimportant(self, entry: EntryInput) -> None:
         """Mark an entry as unimportant.
 
+        Alias for ``set_entry_important(entry, False)``.
+
         Args:
             entry (tuple(str, str) or Entry): (feed URL, entry id) tuple.
 
@@ -1183,8 +1363,105 @@ class Reader:
             Renamed from :meth:`mark_as_unimportant`.
 
         """
+        return self.set_entry_important(entry, False)
+
+    def _mark_entry_as_dont_care(self, entry: EntryInput) -> None:
+        """Mark an entry as read and unimportant at the same time,
+        resulting in the same read_modified and important_modified.
+
+        This method becoming public is pending on #254.
+
+        Presumably, we could just use mark_entry_as_{read,important} instead
+        and get the slightly different timestamps,
+        but it's likely better to collect more accurate data.
+
+        """
+        modified_naive = self._now()
         feed_url, entry_id = _entry_argument(entry)
-        self._storage.mark_as_important_unimportant(feed_url, entry_id, False)
+        self._storage.mark_as_read(feed_url, entry_id, True, modified_naive)
+        self._storage.mark_as_important(feed_url, entry_id, False, modified_naive)
+
+    def add_entry(self, entry: Any) -> None:
+        """Add a new entry to an existing feed.
+
+        ``entry`` can be any :class:`Entry`-like object,
+        or a mapping of the same shape::
+
+            >>> from types import SimpleNamespace
+            >>> reader.add_entry(SimpleNamespace(
+            ...     feed_url='http://example.com',
+            ...     id='one',
+            ...     title='title',
+            ...     enclosures=[SimpleNamespace(href='enclosure')],
+            ... ))
+            >>> reader.add_entry({
+            ...     'feed_url': 'http://example.com',
+            ...     'id': 'two',
+            ...     'updated': datetime.now(timezone.utc),
+            ...     'content': [{'value': 'content'}],
+            ... })
+
+        The following attributes are used
+        (they must have the same types as on :class:`Entry`):
+
+        * :attr:`~Entry.feed_url` (required)
+        * :attr:`~Entry.id` (required)
+        * :attr:`~Entry.updated`
+        * :attr:`~Entry.title`
+        * :attr:`~Entry.link`
+        * :attr:`~Entry.author`
+        * :attr:`~Entry.published`
+        * :attr:`~Entry.summary`
+        * :attr:`~Entry.content`
+        * :attr:`~Entry.enclosures`
+
+        Naive datetimes are normalized by passing them to
+        :meth:`~datetime.datetime.astimezone`.
+
+        The added entry will be :attr:`~Entry.added_by` ``'user'``.
+
+        Args:
+            entry (Entry or dict): An entry-like object or equivalent mapping.
+
+        Raises:
+            EntryExistsError: If an entry with the same id already exists.
+            StorageError
+
+        .. versionadded:: 2.5
+
+        """
+        now = self._now()
+
+        intent = EntryUpdateIntent(
+            entry=entry_data_from_obj(entry),
+            last_updated=now,
+            first_updated=now,
+            first_updated_epoch=now,
+            added_by='user',
+        )
+
+        self._storage.add_entry(intent)
+        for entry_hook in self.after_entry_update_hooks:
+            entry_hook(self, intent.entry, EntryUpdateStatus.NEW)
+
+    def delete_entry(self, entry: EntryInput) -> None:
+        """Delete an entry.
+
+        Currently, only entries added by :meth:`~Reader.add_entry`
+        (:attr:`~Entry.added_by` ``'user'``) can be deleted.
+
+        Args:
+            entry (tuple(str, str) or Entry): (feed URL, entry id) tuple.
+
+        Raises:
+            EntryNotFoundError
+            EntryError: If the entry was not added by the user.
+            StorageError
+
+        .. versionadded:: 2.5
+
+        """
+        self._storage.delete_entries([_entry_argument(entry)], added_by='user')
 
     def get_feed_metadata(
         self,
@@ -1342,13 +1619,22 @@ class Reader:
 
         Search must be enabled to call this method.
 
+        If :func:`make_reader` was called with ``search_enabled='auto'``
+        and search is disabled, it will be enabled automatically.
+
         Raises:
             SearchNotEnabledError
             SearchError
             StorageError
 
         """
-        return self._search.update()
+        try:
+            self._search.update()
+        except SearchNotEnabledError:
+            if not self._enable_search:
+                raise
+            self._search.enable()
+            self._search.update()
 
     def search_entries(
         self,
@@ -1522,7 +1808,8 @@ class Reader:
         filter_options = EntryFilterOptions.from_args(
             feed, entry, read, important, has_enclosures, feed_tags
         )
-        return self._search.search_entry_counts(query, filter_options)
+        now = self._now()
+        return self._search.search_entry_counts(query, now, filter_options)
 
     def add_feed_tag(self, feed: FeedInput, tag: str) -> None:
         """Add a tag to a feed.
@@ -1664,4 +1951,4 @@ class Reader:
         try:
             self._reserved_name_scheme = NameScheme.from_value(value)
         except Exception as e:
-            raise AttributeError(f"invalid scheme: {value}") from e
+            raise AttributeError(f"invalid reserved name scheme: {value}") from e
