@@ -57,7 +57,7 @@ class ReaderError(_FancyExceptionBase):
 
 
 class FeedError(ReaderError):
-    """A feed error occured."""
+    """A feed error occurred."""
 
     def __init__(self, url: str, message: str = '') -> None:
         super().__init__(message)
@@ -91,8 +91,18 @@ class FeedNotFoundError(FeedError):
     message = "no such feed"
 
 
+class InvalidFeedURLError(FeedError, ValueError):
+    """Invalid feed URL.
+
+    .. versionadded:: 2.5
+
+    """
+
+    message = "invalid feed URL"
+
+
 class ParseError(FeedError):
-    """An error occured while getting/parsing feed.
+    """An error occurred while getting/parsing feed.
 
     The original exception should be chained to this one (e.__cause__).
 
@@ -129,6 +139,16 @@ class EntryError(ReaderError):
         return self.feed_url, self.id
 
 
+class EntryExistsError(EntryError):
+    """Entry already exists.
+
+    .. versionadded:: 2.5
+
+    """
+
+    message = "entry exists"
+
+
 class EntryNotFoundError(EntryError):
     """Entry not found."""
 
