@@ -92,6 +92,32 @@ class Feed(_namedtuple_compat):
     #: The author of the feed.
     author: Optional[str] = None
 
+    #: A description or subtitle for the feed.
+    #:
+    #: .. versionadded:: 2.4
+    subtitle: Optional[str] = None
+
+    #: The feed type and version.
+    #:
+    #: For Atom and RSS, provided by `feedparser`_ (e.g. ``atom10``, ``rss20``);
+    #: `full list <https://feedparser.readthedocs.io/en/latest/version-detection.html>`_.
+    #:
+    #: For JSON Feed:
+    #:
+    #: ``json10``
+    #:  `JSON Feed 1.0 <https://www.jsonfeed.org/version/1/>`_
+    #:
+    #: ``json11``
+    #:  `JSON Feed 1.1 <https://www.jsonfeed.org/version/1.1/>`_
+    #:
+    #: ``json``
+    #:  JSON Feed (unknown or unrecognized version)
+    #:
+    #: Plugins may add other versions.
+    #:
+    #: .. versionadded:: 2.4
+    version: Optional[str] = None
+
     #: User-defined feed title.
     user_title: Optional[str] = None
 
@@ -160,6 +186,9 @@ class ExceptionInfo(_namedtuple_compat):
         )
 
 
+EntryAddedBy = Literal['feed', 'user']
+
+
 @dataclass(frozen=True)
 class Entry(_namedtuple_compat):
 
@@ -174,7 +203,7 @@ class Entry(_namedtuple_compat):
 
     """
 
-    # WARNING: When changing attributes, keep Entry and EntryData in sync.
+    # WARNING: When changing attributes, keep Entry, EntryData and entry_data_from_obj in sync.
 
     @property
     def feed_url(self) -> str:
@@ -183,26 +212,21 @@ class Entry(_namedtuple_compat):
 
     # TODO: .id will still be set to some default value if the entry doesn't have it; document this.
 
-    # NOTE: In #183 (2.0 compat breaking), we made .updated optional,
-    # to allow it to reflect the feed value.
-    #
-    # The code to do so hasn't been changed yet,
-    # so its value is actually the same as before.
-    #
-    # To change that, we need to store/add a `first_updated` attribute,
-    # and change `updated_not_none` to `updated or first_updated`.
-
     #: The entry id.
     id: str
 
     #: The date the entry was last updated, according to the feed.
     #:
     #: .. versionchanged:: 2.0
-    #:  May be None in some cases.
-    #:  In a future version, will be None if missing in the feed;
+    #:  May be :const:`None` in some cases.
+    #:  In a future version, will be :const:`None` if missing in the feed;
     #:  use :attr:`updated_not_none` for the pre-2.0 behavior.
     #:
-    updated: Optional[datetime]
+    #: .. versionchanged:: 2.5
+    #:  Is now :const:`None` if missing in the feed;
+    #:  use :attr:`updated_not_none` for the pre-2.5 behavior.
+    #:
+    updated: Optional[datetime] = None
 
     #: The title of the entry.
     title: Optional[str] = None
@@ -213,7 +237,7 @@ class Entry(_namedtuple_compat):
     #: The author of the feed.
     author: Optional[str] = None
 
-    #: The date the entry was first published.
+    #: The date the entry was published, according to the feed.
     published: Optional[datetime] = None
 
     #: A summary of the entry.
@@ -230,9 +254,35 @@ class Entry(_namedtuple_compat):
     #: Whether the entry was read or not.
     read: bool = False
 
+    #: The date when :attr:`read` was last set by the user;
+    #: :const:`None` if that never happened,
+    #: or the entry predates the date being recorded.
+    #:
+    #: .. versionadded:: 2.2
+    read_modified: Optional[datetime] = None
+
     #: Whether the entry is important or not.
     important: bool = False
 
+    #: The date when :attr:`important` was last set by the user;
+    #: :const:`None` if that never happened,
+    #: or the entry predates the date being recorded.
+    #:
+    #: .. versionadded:: 2.2
+    important_modified: Optional[datetime] = None
+
+    #: The date when the entry was added (first updated) to reader.
+    #:
+    #: .. versionadded:: 2.5
+    added: datetime = cast(datetime, None)
+
+    #: The source of the entry. One of ``'feed'``, ``'user'``.
+    #:
+    #: Other values may be added in the future.
+    #:
+    #: .. versionadded:: 2.5
+    added_by: EntryAddedBy = cast(EntryAddedBy, None)
+
     #: The date when the entry was last updated by reader.
     #:
     #: .. versionadded:: 1.3
@@ -271,15 +321,13 @@ class Entry(_namedtuple_compat):
         """Like :attr:`updated`, but guaranteed to be set (not None).
 
         If the entry `updated` is missing in the feed,
-        defaults to when the entry was first added.
+        defaults to when the entry was first `added`.
 
         .. versionadded:: 2.0
             Identical to the behavior of :attr:`updated` before 2.0.
 
         """
-        # for mypy
-        assert self.updated is not None
-        return self.updated
+        return self.updated or self.added
 
 
 @dataclass(frozen=True)
@@ -287,6 +335,8 @@ class Content(_namedtuple_compat):
 
     """Data type representing a piece of content."""
 
+    # WARNING: When changing attributes, keep content_from_obj in sync.
+
     #: The content value.
     value: str
 
@@ -302,6 +352,8 @@ class Enclosure(_namedtuple_compat):
 
     """Data type representing an external file."""
 
+    # WARNING: When changing attributes, keep content_from_obj in sync.
+
     #: The file URL.
     href: str
 
@@ -664,6 +716,15 @@ class EntryCounts(_namedtuple_compat):
     #: Number of entries that have enclosures.
     has_enclosures: Optional[int] = None
 
+    # TODO: make averages a rich tuple
+    # https://github.com/lemon24/reader/issues/249#issuecomment-894624989
+
+    #: Average entries per day during the last 1, 3, 12 months, as a 3-tuple.
+    #:
+    #: .. versionadded:: 2.1
+    #:
+    averages: Optional[Tuple[float, float, float]] = None
+
 
 @dataclass(frozen=True)
 class EntrySearchCounts(_namedtuple_compat):
@@ -693,6 +754,12 @@ class EntrySearchCounts(_namedtuple_compat):
     #: Number of entries that have enclosures.
     has_enclosures: Optional[int] = None
 
+    #: Average entries per day during the last 1, 3, 12 months, as a 3-tuple.
+    #:
+    #: .. versionadded:: 2.1
+    #:
+    averages: Optional[Tuple[float, float, float]] = None
+
 
 @dataclass(frozen=True)
 class UpdatedFeed:
@@ -737,7 +804,7 @@ class UpdateResult(NamedTuple):
     #: :obj:`None`
     #:
     #:  If the server indicated the feed has not changed
-    #:  since the last update.
+    #:  since the last update without returning any data.
     #:
     #: :exc:`ReaderError`
     #:
@@ -752,3 +819,37 @@ class UpdateResult(NamedTuple):
     # Currently, storage or plugin-raised exceptions
     # prevent updates for the following feeds (:issue:`218`),
     # but that's not necessarily by design.
+
+    @property
+    def updated_feed(self) -> Optional[UpdatedFeed]:
+        """The updated feed, if the update was successful, :const:`None` otherwise.
+
+        .. versionadded:: 2.1
+
+        """
+        return self.value if not isinstance(self.value, Exception) else None
+
+    @property
+    def error(self) -> Optional[ReaderError]:
+        """The exception, if there was an error, :const:`None` otherwise.
+
+        .. versionadded:: 2.1
+
+        """
+        return self.value if isinstance(self.value, Exception) else None
+
+    @property
+    def not_modified(self) -> bool:
+        """True if the feed has not changed
+        (either because the server returned no data,
+        or because the data didn't change),
+        false otherwise.
+
+        .. versionadded:: 2.1
+
+        """
+        if self.error:
+            return False
+        if not self.updated_feed:
+            return True
+        return not (self.updated_feed.new or self.updated_feed.modified)
