@@ -1,3 +1,4 @@
+import os
 import sqlite3
 import sys
 from contextlib import closing
@@ -5,6 +6,7 @@ from functools import wraps
 
 import py.path
 import pytest
+from utils import monkeypatch_tz
 from utils import reload_module
 
 from reader import make_reader as original_make_reader
@@ -47,6 +49,18 @@ def apply_flaky_pypy_sqlite3(items):  # pragma: no cover
         item.add_marker(sqlite3_flaky)
 
 
+def pytest_runtest_setup(item):
+    # lxml fails to build in various places,
+    # see the comments in setup.py for details.
+    for mark in item.iter_markers(name="requires_lxml"):
+        no_lxml = [
+            sys.implementation.name == 'pypy',
+            os.name == 'nt' and sys.version_info[:2] == (3, 10),
+        ]
+        if any(no_lxml):
+            pytest.skip("test requires lxml")
+
+
 @pytest.fixture
 def make_reader(request):
     @wraps(original_make_reader)
