@@ -33,7 +33,9 @@ def test_closed(reader):
 
 
 @pytest.fixture(params=[False, True], ids=['without_entries', 'with_entries'])
-def reader_without_and_with_entries(request, reader):
+def reader_without_and_with_entries(request, make_reader):
+    reader = make_reader(':memory:', search_enabled=None)
+
     if not request.param:
         return reader
 
@@ -60,6 +62,9 @@ def reader_without_and_with_entries(request, reader):
     return reader
 
 
+with_sort = pytest.mark.parametrize('sort', ['relevant', 'recent'])
+
+
 @rename_argument('reader', 'reader_without_and_with_entries')
 def test_search_disabled_by_default(reader):
     assert not reader.is_search_enabled()
@@ -96,7 +101,70 @@ def test_update_search(reader):
     reader.update_search()
 
 
-with_sort = pytest.mark.parametrize('sort', ['relevant', 'recent'])
+@rename_argument('reader', 'reader_without_and_with_entries')
+def test_update_search_fails_if_not_enabled(reader):
+    with pytest.raises(SearchNotEnabledError) as excinfo:
+        reader.update_search()
+    assert excinfo.value.__cause__ is None
+    assert excinfo.value.message
+
+
+@rename_argument('reader', 'reader_without_and_with_entries')
+@with_sort
+def test_search_entries_fails_if_not_enabled(reader, sort):
+    with pytest.raises(SearchNotEnabledError) as excinfo:
+        list(reader.search_entries('one', sort=sort))
+    assert excinfo.value.__cause__ is None
+    assert excinfo.value.message
+
+
+@rename_argument('reader', 'reader_without_and_with_entries')
+def test_search_entry_counts_fails_if_not_enabled(reader):
+    with pytest.raises(SearchNotEnabledError) as excinfo:
+        list(reader.search_entry_counts('one'))
+    assert excinfo.value.__cause__ is None
+    assert excinfo.value.message
+
+
+def test_search_enabled_true(make_reader):
+    reader = make_reader(':memory:', search_enabled=True)
+    assert reader.is_search_enabled()
+
+    reader.update_search()
+    list(reader.search_entries('one'))
+
+
+def test_search_enabled_false(make_reader, db_path):
+    reader = make_reader(db_path, search_enabled=None)
+    reader.enable_search()
+    assert reader.is_search_enabled()
+
+    reader = make_reader(db_path, search_enabled=False)
+    assert not reader.is_search_enabled()
+
+    with pytest.raises(SearchNotEnabledError):
+        reader.update_search()
+    with pytest.raises(SearchNotEnabledError):
+        list(reader.search_entries('one'))
+
+
+@pytest.mark.parametrize('kwargs', [{}, dict(search_enabled='auto')])
+def test_search_enabled_auto(make_reader, kwargs):
+    reader = make_reader(':memory:', **kwargs)
+    assert not reader.is_search_enabled()
+
+    with pytest.raises(SearchNotEnabledError):
+        list(reader.search_entries('one'))
+
+    reader.update_search()
+    list(reader.search_entries('one'))
+
+
+@pytest.mark.parametrize('search_enabled', ['a', 2])
+def test_search_enabled_value_error(make_reader, search_enabled):
+    with pytest.raises(ValueError) as excinfo:
+        make_reader(':memory:', search_enabled=search_enabled)
+    assert 'search_enabled' in str(excinfo.value)
 
 
 @rename_argument('reader', 'reader_without_and_with_entries')
@@ -432,35 +500,12 @@ def test_update_triggers_no_change(db_path, make_reader, monkeypatch, set_user_t
     assert strip_html_called == 0
 
 
-@rename_argument('reader', 'reader_without_and_with_entries')
-def test_update_search_fails_if_not_enabled(reader):
-    with pytest.raises(SearchNotEnabledError) as excinfo:
-        reader.update_search()
-    assert excinfo.value.__cause__ is None
-    assert excinfo.value.message
-
-
-@rename_argument('reader', 'reader_without_and_with_entries')
-@with_sort
-def test_search_entries_fails_if_not_enabled(reader, sort):
-    with pytest.raises(SearchNotEnabledError) as excinfo:
-        list(reader.search_entries('one', sort=sort))
-    assert excinfo.value.__cause__ is None
-    assert excinfo.value.message
-
-
-@rename_argument('reader', 'reader_without_and_with_entries')
-def test_search_entry_counts_fails_if_not_enabled(reader):
-    with pytest.raises(SearchNotEnabledError) as excinfo:
-        list(reader.search_entry_counts('one'))
-    assert excinfo.value.__cause__ is None
-    assert excinfo.value.message
-
-
 @with_sort
 def test_search_entries_basic(reader, sort):
     parser = Parser()
     reader._parser = parser
+    # we're far intro the future, there are no recent entries
+    reader._now = lambda: naive_datetime(2020, 1, 1)
 
     feed = parser.feed(1, datetime(2010, 1, 1))
     one = parser.entry(1, 1, datetime(2010, 1, 1), title='one')
@@ -494,7 +539,7 @@ def test_search_entries_basic(reader, sort):
     # TODO: the asserts below look parametrizable
 
     assert list(search('zero')) == []
-    assert search_counts('zero') == EntrySearchCounts(0, 0, 0, 0)
+    assert search_counts('zero') == EntrySearchCounts(0, 0, 0, 0, (0, 0, 0))
     assert list(search('one')) == [
         EntrySearchResult(
             feed.url,
@@ -505,7 +550,7 @@ def test_search_entries_basic(reader, sort):
             },
         )
     ]
-    assert search_counts('one') == EntrySearchCounts(1, 0, 0, 0)
+    assert search_counts('one') == EntrySearchCounts(1, 0, 0, 0, (0, 0, 0))
     assert list(search('two')) == [
         EntrySearchResult(
             feed.url,
@@ -578,12 +623,51 @@ def test_search_entries_basic(reader, sort):
             ),
         ]
     }
-    assert search_counts('summary') == EntrySearchCounts(3, 0, 0, 0)
+    assert search_counts('summary') == EntrySearchCounts(3, 0, 0, 0, (0, 0, 0))
 
 
 # search_entries() filtering is tested in test_reader.py::test_entries_filtering{,_error}
 
 
+from test_reader import entries_per_day
+
+
+@pytest.mark.parametrize(
+    'query, expected',
+    [
+        ('one', entries_per_day(0, 0, 1)),
+        ('two', entries_per_day(0, 1, 1)),
+        ('three', entries_per_day(1, 1, 1)),
+        ('entry', entries_per_day(1, 2, 2)),
+        ('one OR two', entries_per_day(0, 1, 2)),
+        ('one OR three', entries_per_day(1, 1, 2)),
+        ('one OR entry', entries_per_day(1, 2, 3)),
+        ('feed', entries_per_day(1, 2, 3)),
+        ('four', (0, 0, 0)),
+    ],
+)
+def test_search_entry_counts_basic(reader, query, expected):
+    # search_entry_counts() filtering is tested in test_reader.py::test_entry_counts
+
+    parser = Parser()
+    reader._parser = parser
+
+    feed = parser.feed(1, datetime(2010, 1, 1))
+    parser.entry(1, 1, datetime(2010, 2, 15), title='one')
+    parser.entry(1, 2, datetime(2010, 11, 15), title='entry two')
+    parser.entry(1, 3, datetime(2010, 12, 16), title='entry three')
+
+    reader.add_feed(feed.url)
+    reader.update_feeds()
+
+    reader.enable_search()
+    reader.update_search()
+
+    reader._now = lambda: naive_datetime(2010, 12, 31)
+
+    assert reader.search_entry_counts(query).averages == expected
+
+
 # TODO: fix duplication in these order tests
 # BEGIN order tests
 
@@ -985,3 +1069,38 @@ def test_update_search_concurrent_calls(db_path, make_reader, monkeypatch):
 
 
 # END concurrency tests
+
+
+def test_add_entry_repeatedly(reader):
+    # this could have been found with Hypothesis
+
+    reader.enable_search()
+    reader.add_feed('1')
+
+    # we should not get
+    #   sqlite3.IntegrityError: UNIQUE constraint failed:
+    #     entries_search_sync_state.id, entries_search_sync_state.feed
+    # on the second loop
+
+    for _ in range(3):
+        reader.add_entry(dict(feed_url='1', id='1'))
+        reader.delete_entry(('1', '1'))
+
+
+def test_add_entry_basic(reader):
+    reader.enable_search()
+    reader.add_feed('1')
+    reader.add_entry(
+        dict(feed_url='1', id='1', title='my entry', summary='I am a summary')
+    )
+    reader.update_search()
+
+    (result,) = reader.search_entries('entry')
+    assert result.object_id == ('1', '1')
+    assert result.metadata['.title'].apply('*', '*') == 'my *entry*'
+    assert result.content['.summary'].apply('*', '*') == 'I am a summary'
+
+    (result,) = reader.search_entries('summary')
+    assert result.object_id == ('1', '1')
+    assert result.metadata['.title'].apply('*', '*') == 'my entry'
+    assert result.content['.summary'].apply('*', '*') == 'I am a *summary*'
