@@ -197,8 +197,12 @@ def mark_as_stale(storage, feed, __):
     storage.mark_as_stale(feed.url)
 
 
-def mark_as_read_unread(storage, feed, entry):
-    storage.mark_as_read_unread(feed.url, entry.id, 1)
+def mark_as_read(storage, feed, entry):
+    storage.mark_as_read(feed.url, entry.id, 1, None)
+
+
+def mark_as_important(storage, feed, entry):
+    storage.mark_as_important(feed.url, entry.id, 1, None)
 
 
 def update_feed(storage, feed, entry):
@@ -211,16 +215,37 @@ def update_feed_last_updated(storage, feed, entry):
 
 def add_or_update_entry(storage, feed, entry):
     storage.add_or_update_entry(
-        EntryUpdateIntent(entry, entry.updated, datetime(2010, 1, 1), 0, 0)
+        EntryUpdateIntent(
+            entry, entry.updated, datetime(2010, 1, 1), datetime(2010, 1, 1)
+        )
     )
 
 
 def add_or_update_entries(storage, feed, entry):
     storage.add_or_update_entries(
-        [EntryUpdateIntent(entry, entry.updated, datetime(2010, 1, 1), 0, 0)]
+        [
+            EntryUpdateIntent(
+                entry,
+                entry.updated,
+                datetime(2010, 1, 1),
+                datetime(2010, 1, 1),
+            )
+        ]
     )
 
 
+def add_entry(storage, feed, entry):
+    storage.add_entry(
+        EntryUpdateIntent(
+            entry, entry.updated, datetime(2010, 1, 1), datetime(2010, 1, 1)
+        )
+    )
+
+
+def delete_entries(storage, feed, entry):
+    storage.delete_entries([entry.object_id])
+
+
 def get_entries_chunk_size_0(storage, _, __):
     list(storage.get_entries_page(chunk_size=0, now=datetime(2010, 1, 1)))
 
@@ -258,7 +283,7 @@ def get_feed_counts(storage, _, __):
 
 
 def get_entry_counts(storage, _, __):
-    storage.get_entry_counts(),
+    storage.get_entry_counts(now=datetime(2010, 1, 1)),
 
 
 def get_feed_last(storage, feed, __):
@@ -282,11 +307,14 @@ def get_entry_last(storage, feed, entry):
         set_feed_user_title,
         set_feed_updates_enabled,
         mark_as_stale,
-        mark_as_read_unread,
+        mark_as_read,
+        mark_as_important,
         update_feed,
         update_feed_last_updated,
         add_or_update_entry,
         add_or_update_entries,
+        add_entry,
+        delete_entries,
         get_entries_chunk_size_0,
         get_entries_chunk_size_1,
         iter_metadata,
@@ -318,7 +346,9 @@ def check_errors_locked(db_path, pre_stuff, do_stuff, exc_type):
     entry = EntryData('one', 'entry', datetime(2010, 1, 2))
     storage.add_feed(feed.url, datetime(2010, 1, 1))
     storage.add_or_update_entry(
-        EntryUpdateIntent(entry, entry.updated, datetime(2010, 1, 1), 0, 0)
+        EntryUpdateIntent(
+            entry, entry.updated, datetime(2010, 1, 1), datetime(2010, 1, 1), 0, 0
+        )
     )
 
     in_transaction = threading.Event()
@@ -414,12 +444,19 @@ def check_iter_locked(db_path, pre_stuff, iter_stuff):
     entry = EntryData('one', 'entry', datetime(2010, 1, 1), title='entry')
     storage.add_feed(feed.url, datetime(2010, 1, 2))
     storage.add_or_update_entry(
-        EntryUpdateIntent(entry, entry.updated, datetime(2010, 1, 1), 0, 0)
+        EntryUpdateIntent(
+            entry, entry.updated, datetime(2010, 1, 1), datetime(2010, 1, 1), 0, 0
+        )
     )
     storage.add_feed('two', datetime(2010, 1, 1))
     storage.add_or_update_entry(
         EntryUpdateIntent(
-            entry._replace(feed_url='two'), entry.updated, datetime(2010, 1, 1), 0, 0
+            entry._replace(feed_url='two'),
+            entry.updated,
+            datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
+            0,
+            0,
         )
     )
     storage.set_metadata(('two',), '1', 1)
@@ -435,9 +472,9 @@ def check_iter_locked(db_path, pre_stuff, iter_stuff):
 
     # shouldn't raise an exception
     storage = Storage(db_path, timeout=0, wal_enabled=False)
-    storage.mark_as_read_unread(feed.url, entry.id, 1)
+    storage.mark_as_read(feed.url, entry.id, 1, None)
     storage = Storage(db_path, timeout=0)
-    storage.mark_as_read_unread(feed.url, entry.id, 0)
+    storage.mark_as_read(feed.url, entry.id, 0, None)
 
 
 def test_update_feed_last_updated_not_found(db_path):
@@ -507,13 +544,14 @@ def test_get_entries_for_update(storage_cls):
             entry,
             datetime(2010, 1, 2),
             datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
             0,
             0,
         )
     )
 
     assert list(storage.get_entries_for_update([('feed', 'one'), ('feed', 'two')])) == [
-        EntryForUpdate(datetime(2010, 1, 1), entry.hash, 0),
+        EntryForUpdate(datetime(2010, 1, 1), None, entry.hash, 0),
         None,
     ]
 
@@ -523,28 +561,6 @@ def storage():
     return Storage(':memory:')
 
 
-def test_entry_remains_read_after_update(storage_with_two_entries):
-    storage = storage_with_two_entries
-    storage.mark_as_read_unread('feed', 'one', True)
-
-    storage.add_or_update_entry(
-        EntryUpdateIntent(
-            EntryData('feed', 'one', datetime(2010, 1, 1)),
-            datetime(2010, 1, 2),
-            datetime(2010, 1, 2),
-            0,
-            0,
-        )
-    )
-
-    assert {
-        e.id
-        for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(read=True)
-        )
-    } == {'one'}
-
-
 @pytest.fixture
 def storage_with_two_entries(storage):
     storage.add_feed('feed', datetime(2010, 1, 1))
@@ -553,6 +569,7 @@ def storage_with_two_entries(storage):
             EntryData('feed', 'one', datetime(2010, 1, 1)),
             datetime(2010, 1, 2),
             datetime(2010, 1, 2),
+            datetime(2010, 1, 2),
             0,
             0,
         )
@@ -562,6 +579,7 @@ def storage_with_two_entries(storage):
             EntryData('feed', 'two', datetime(2010, 1, 1)),
             datetime(2010, 1, 2),
             datetime(2010, 1, 2),
+            datetime(2010, 1, 2),
             1,
             0,
         )
@@ -581,7 +599,7 @@ def test_important_unimportant_by_default(storage):
 
 @rename_argument('storage', 'storage_with_two_entries')
 def test_important_get_entries(storage):
-    storage.mark_as_important_unimportant('feed', 'one', True)
+    storage.mark_as_important('feed', 'one', True, datetime(2010, 1, 2))
 
     assert {e.id for e in storage.get_entries(now=datetime(2010, 1, 1))} == {
         'one',
@@ -607,31 +625,9 @@ def test_important_get_entries(storage):
     } == {'two'}
 
 
-@rename_argument('storage', 'storage_with_two_entries')
-def test_important_entry_remains_important_after_update(storage):
-    storage.mark_as_important_unimportant('feed', 'one', True)
-
-    storage.add_or_update_entry(
-        EntryUpdateIntent(
-            EntryData('feed', 'one', datetime(2010, 1, 1)),
-            datetime(2010, 1, 2),
-            datetime(2010, 1, 2),
-            0,
-            0,
-        )
-    )
-
-    assert {
-        e.id
-        for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important=True)
-        )
-    } == {'one'}
-
-
 @rename_argument('storage', 'storage_with_two_entries')
 def test_important_entry_important(storage):
-    storage.mark_as_important_unimportant('feed', 'one', True)
+    storage.mark_as_important('feed', 'one', True, None)
 
     assert {e.id: e.important for e in storage.get_entries(datetime(2010, 1, 1))} == {
         'one': True,
@@ -641,8 +637,8 @@ def test_important_entry_important(storage):
 
 @rename_argument('storage', 'storage_with_two_entries')
 def test_important_mark_as_unimportant(storage):
-    storage.mark_as_important_unimportant('feed', 'one', True)
-    storage.mark_as_important_unimportant('feed', 'one', False)
+    storage.mark_as_important('feed', 'one', True, None)
+    storage.mark_as_important('feed', 'one', False, None)
 
     assert {
         e.id
@@ -654,7 +650,7 @@ def test_important_mark_as_unimportant(storage):
 
 def test_important_mark_entry_not_found(storage):
     with pytest.raises(EntryNotFoundError):
-        storage.mark_as_important_unimportant('feed', 'one', True)
+        storage.mark_as_important('feed', 'one', True, None)
 
 
 def test_minimum_sqlite_version(db_path, monkeypatch):
