@@ -2,6 +2,7 @@ import functools
 import importlib
 import inspect
 import os.path
+import time
 from datetime import datetime
 from datetime import timezone
 from urllib.parse import urlparse
@@ -75,6 +76,34 @@ def reload_module(monkeypatch):
         reloader.undo()
 
 
+class TZSetter:
+    def __init__(self, monkeypatch):
+        self.monkeypatch = monkeypatch
+
+    def __call__(self, tz):
+        self.monkeypatch.setenv('TZ', tz)
+        time.tzset()
+
+    def undo(self):
+        self.monkeypatch.undo()
+        time.tzset()
+
+
+@pytest.fixture
+def monkeypatch_tz(monkeypatch):
+    tzsetter = TZSetter(monkeypatch)
+    try:
+        yield tzsetter
+    finally:
+        try:
+            tzsetter.undo()
+        except AttributeError as e:
+            # on windows, we get "module 'time' has no attribute 'tzset'";
+            # it's ok to do nothing, since  __call__() didn't call it either
+            if 'tzset' not in str(e):
+                raise
+
+
 # FIXME: explain what this is
 # https://github.com/lemon24/reader/issues/233
 
